#!/usr/bin/env python
# *-* coding: utf-8 *-*
"""Plot a wireframe of a given grid
"""
import os
from optparse import OptionParser
import numpy as np
from crtomo.mpl_setup import *
import crtomo.grid as CRGrid

# # environment variables
#
node_mark_size = float(os.environ.get('MARK_SIZE', 1.0))
# line width
cell_mark_size = float(os.environ.get('MARK_WIDTH', 1.0))
cell_line_width = float(os.environ.get('CELL_WIDTH', 1.0))
# electrode size
elec_size = float(os.environ.get('ELEC_SIZE', 30.0))
dpi = int(os.environ.get('DPI', 300))


def handle_cmd_options():
    parser = OptionParser()
    parser.add_option(
        '-e', "--elem",
        dest="elem_file",
        type="string",
        help="elem.dat file (default: elem.dat)",
        default="elem.dat",
    )

    parser.add_option(
        '-t', "--elec",
        dest="elec_file",
        type="string",
        help="elec.dat file (default: elec.dat)",
        default="elec.dat",
    )

    parser.add_option(
        '-d', "--decouplings",
        dest="decoupling_file",
        type="string",
        help='decouplings file (default: ../exe/decouplings.dat)',
        default="../exe/decouplings.dat",
    )

    parser.add_option(
        "-o", "--output",
        dest="output",
        help="Output file (default: grid.png)",
        metavar="FILE",
        default="grid.png",
    )

    parser.add_option(
        "-m", "--mark_node",
        dest="mark_node",
        help="Mark one node (index starts with 0)",
        type="int",
        metavar="NR",
        default=None,
    )

    parser.add_option(
        "-c", "--mark_cell",
        dest="mark_cell",
        help="Mark one cell (index starts with 0)",
        type="int",
        metavar="NR",
        default=None,
    )
    parser.add_option(
        "--fancy",
        action="store_true",
        dest="plot_fancy",
        help="Create a fancy plot (default: True)",
        default=True,
    )
    parser.add_option(
        '-n',
        "--plot_elec_nr",
        action="store_true",
        dest="plot_elec_nr",
        help="Plot electrode numbers next to the electrodes",
        default=False,
    )
    (options, args) = parser.parse_args()
    return options


def plot_wireframe(options):
    grid = CRGrid.crt_grid()
    grid.load_elem_file(options.elem_file)
    grid.load_elec_file(options.elec_file)

    xmin = grid.grid['x'].min()
    xmax = grid.grid['x'].max()
    zmin = grid.grid['z'].min()
    zmax = grid.grid['z'].max()

    fig, ax = plt.subplots(1, 1, frameon=False)
    all_xz = []
    for x, z in zip(grid.grid['x'], grid.grid['z']):
        tmp = np.vstack((x, z)).T
        all_xz.append(tmp)
    collection = mpl.collections.PolyCollection(
        all_xz,
        edgecolor='k',
        facecolor='none',
        linewidth=cell_line_width,
    )
    ax.add_collection(collection)

    # plot electrodes
    ax.scatter(
        grid.electrodes[:, 1],
        grid.electrodes[:, 2],
        edgecolors='none',
        clip_on=False,
        label='electrodes',
        s=elec_size
    )

    if options.plot_elec_nr:
        for nr, xy in enumerate(grid.electrodes[:, 1:3]):
            ax.text(
                xy[0], xy[1],
                '{}'.format(nr + 1),
                bbox=dict(boxstyle='circle', facecolor='red', alpha=0.8)
            )

    # mark nodes
    if options.mark_node is not None:
        xy = grid.nodes['sorted'][options.mark_node]
        ax.scatter(
            xy[1],
            xy[2],
            s=node_mark_size,
            color='r',
            edgecolors='none',
            label='marked node',
        )

    if options.mark_cell is not None:
        index = options.mark_cell
        x = grid.grid['x'][index]
        z = grid.grid['z'][index]
        label = 'marked cell'
        for i in range(0, x.size):
            i1 = (i + 1) % x.size
            ax.plot(
                x[[i, i1]],
                z[[i, i1]],
                color='r',
                linewidth=cell_mark_size,
                label=label,
            )
            label = ''

        polygon = mpl.patches.Polygon(
            [(a, b) for a, b in zip(x, z)],
            closed=True,
            color='r',
            alpha=1.0,
        )
        ax.add_patch(polygon)

    if os.path.isfile(options.decoupling_file):
        decouplings = np.loadtxt(options.decoupling_file, skiprows=1)
        # plot decouplings
        label = 'decoupling line'
        for (el1, el2, coef) in decouplings:
            n1 = grid.elements[int(el1) - 1]
            n2 = grid.elements[int(el2) - 1]

            ints = np.intersect1d(n1, n2)

            x = grid.nodes['presort'][ints, 1]
            z = grid.nodes['presort'][ints, 2]

            ax.plot(
                x,
                z,
                '.-',
                color='b',
                linestyle='dashed',
                label=label,
            )
            label = ''

    ax.autoscale_view()
    ax.set_aspect('equal')
    if options.plot_fancy:
        ax.set_xlabel('x [m]')
        ax.set_ylabel('z [m]')
        ax.legend(
            loc="lower center",
            ncol=4,
            bbox_to_anchor=(0, 0, 1, 1),
            bbox_transform=fig.transFigure
        )

    else:
        ax.axis('off')
        ax.xaxis.set_visible(False)
        ax.yaxis.set_visible(False)

    ax.set_xlim(xmin, xmax)
    ax.set_ylim(zmin, zmax)

    fig.savefig(options.output, dpi=dpi, bbox_inches='tight')


def main():
    options = handle_cmd_options()
    plot_wireframe(options)



#!/usr/bin/env python
# *-* coding: utf-8 *-Ü
"""For a regular grid, create decouplings along a horizontal line in the grid.

Two CMD parameters are required:

    1) depth (in elements) at which the line is located. The first line is 1.
    2) grid width in element numbers


Example:

    >>> grid_gen_decouplings 2 5

    Generate a decoupling line in 2m depths, with the grid consisting of 5
    elements in x direction.


Usage
-----

Pipe the output directly to a decouplings.dat file: ::

    echo "40" > decouplings.dat && grid_gen_decouplings.py 5 40 >> \
    decouplings.dat

The first line denotes the number of decouplings.

Chain multiple calls to decouple multiple lines ::

    echo "80" > decouplings.dat && grid_gen_decouplings.py 5 40 >> \
    decouplings.dat && grid_gen_decouplings.py 8 40 >> \
    decouplings.dat

"""
import sys


def main():
    depth = int(sys.argv[1]) - 1
    width = int(sys.argv[2])

    if depth <= 0:
        raise Exception("First parameter must be larger than 1!")

    assert(depth > 0)

    start = depth * width + 1
    for i in range(start, start + width):
        print('{0} {1} 0'.format(i, i + width))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
'''
Tool to plot inversion results of tomodir. Included data is
* magnitude
* coverage
* phase
* phase of FPI
* real part
* real part of FPI
* imaginary part
* imaginary part of FPI

The three main options are to plot everything in one figure, to plot individual
figures (--single) or to plot anisotropic results of magnitude and phase
(--aniso).
The script has to be run in a tomodir. Output file will be saved in tomodir.

'''
import os
from optparse import OptionParser
import numpy as np

import crtomo.mpl
import crtomo.mpl as mpl_style
import matplotlib
import matplotlib.cm as mpl_cm
# import matplotlib.pyplot as plt
# import matplotlib as mpl

import crtomo.plotManager as CRPlot
import crtomo.grid as CRGrid
import reda.main.units as units
plt, mpl = crtomo.mpl.setup()


def handle_options():
    '''Handle options.
    '''
    parser = OptionParser()
    parser.set_defaults(cmaglin=False)
    parser.set_defaults(single=False)
    parser.set_defaults(aniso=False)
    parser.set_defaults(hlam=False)
    parser.set_defaults(alpha_cov=False)
    # general options
    parser.add_option("--single",
                      action="store_true",
                      dest="single",
                      help="plot each value into a separate file",
                      )
    parser.add_option("--aniso",
                      action="store_true",
                      dest="aniso",
                      help="plot anisotropic xyz",
                      )
    parser.add_option("--hlam",
                      action="store_true",
                      dest="hlam",
                      help="plot anisotropic hor/ver",
                      )
    parser.add_option('--no_elecs',
                      action='store_true',
                      dest='no_elecs',
                      help='Plot no electrodes (default: false)',
                      default=False,
                      )
    parser.add_option('--title',
                      dest='title',
                      type='string',
                      help='Global override for title',
                      default=None,
                      )
    parser.add_option("--alpha_cov",
                      action="store_true",
                      dest="alpha_cov",
                      help="use coverage for transparency",
                      )
    parser.add_option('-c',
                      '--column',
                      dest='column',
                      help='column to plot of input file',
                      type='int',
                      default=2,
                      )
    parser.add_option("--cmaglin",
                      action="store_true",
                      dest="cmaglin",
                      help="linear colorbar for magnitude",
                      )
    # geometric options
    parser.add_option('-x',
                      '--xmin',
                      dest='xmin',
                      help='Minium X range',
                      type='float',
                      )
    parser.add_option('-X',
                      '--xmax',
                      dest='xmax',
                      help='Maximum X range',
                      type='float',
                      )
    parser.add_option('-z',
                      '--zmin',
                      dest='zmin',
                      help='Minium Z range',
                      type='float',
                      )
    parser.add_option('-Z',
                      '--zmax',
                      dest='zmax',
                      help='Maximum Z range',
                      type='float',
                      )
    parser.add_option('-u',
                      '--unit',
                      dest='unit',
                      help='Unit of length scale, typically meters (m) '
                      'or centimeters (cm)',
                      metavar='UNIT',
                      type='str',
                      default='m',
                      )
    # options for colorbars
    parser.add_option('--cov_cbtiks',
                      dest='cov_cbtiks',
                      help="Number of CB tiks for coverage",
                      type=int,
                      metavar="INT",
                      default=3,
                      )
    parser.add_option('--cov_vmin',
                      dest='cov_vmin',
                      help='Minium of colorbar',
                      type='float',
                      )
    parser.add_option('--cov_vmax',
                      dest='cov_vmax',
                      help='Maximum of colorbar',
                      type='float',
                      )
    parser.add_option('--mag_cbtiks',
                      dest='mag_cbtiks',
                      help="Number of CB tiks for magnitude",
                      type=int,
                      metavar="INT",
                      default=3,
                      )
    parser.add_option('--mag_vmin',
                      dest='mag_vmin',
                      help='Minium of colorbar',
                      type='float',
                      )
    parser.add_option('--mag_vmax',
                      dest='mag_vmax',
                      help='Maximum of colorbar',
                      type='float',
                      )
    parser.add_option('--pha_cbtiks',
                      dest='pha_cbtiks',
                      help="Number of CB tiks for phase",
                      type=int,
                      metavar="INT",
                      default=3,
                      )
    parser.add_option('--pha_vmin',
                      dest='pha_vmin',
                      help='Minium of colorbar',
                      type='float',
                      )
    parser.add_option('--pha_vmax',
                      dest='pha_vmax',
                      help='Maximum of colorbar',
                      type='float',
                      )
    parser.add_option('--real_cbtiks',
                      dest='real_cbtiks',
                      help="Number of CB tiks for real part",
                      type=int,
                      metavar="INT",
                      default=3,
                      )
    parser.add_option('--real_vmin',
                      dest='real_vmin',
                      help='Minium of colorbar',
                      type='float',
                      )
    parser.add_option('--real_vmax',
                      dest='real_vmax',
                      help='Maximum of colorbar',
                      type='float',
                      )
    parser.add_option('--imag_cbtiks',
                      dest='imag_cbtiks',
                      help="Number of CB tiks for imag part",
                      type=int,
                      metavar="INT",
                      default=3,
                      )
    parser.add_option('--imag_vmin',
                      dest='imag_vmin',
                      help='Minium of colorbar',
                      type='float',
                      )
    parser.add_option('--imag_vmax',
                      dest='imag_vmax',
                      help='Maximum of colorbar',
                      type='float',
                      )
    parser.add_option('--rat_vmin',
                      dest='rat_vmin',
                      help='Minium of colorbar',
                      type='float',
                      )
    parser.add_option('--rat_vmax',
                      dest='rat_vmax',
                      help='Maximum of colorbar',
                      type='float',
                      )

    parser.add_option(
        "--clog",
        action="store_true",
        dest="c_in_log",
        help="Plot real and imaginary part of conductivity in log10",
        # default=True,
    )

    (options, args) = parser.parse_args()
    return options


def read_iter(use_fpi):
    '''Return the path to the final .mag file either for the complex or the fpi
    inversion.
    '''
    filename_rhosuffix = 'exe/inv.lastmod_rho'
    filename = 'exe/inv.lastmod'
    # filename HAS to exist. Otherwise the inversion was not finished
    if(not os.path.isfile(filename)):
        print('Inversion was not finished! No last iteration found.')
        return 0

    if(use_fpi is True):
        if(os.path.isfile(filename_rhosuffix)):
            filename = filename_rhosuffix

    linestring = open(filename, 'r').readline().strip()
    linestring = linestring.replace('\n', '')
    linestring = linestring.replace('../', '')
    return linestring


def td_type():
    '''get type of the tomodir (complex or dc and whether fpi)
    '''
    cfg = np.genfromtxt('exe/crtomo.cfg',
                        skip_header=15,
                        dtype='str',
                        usecols=([0]))
    is_complex = False
    if cfg[0] == 'F':
        is_complex = True
    is_fpi = False
    if cfg[2] == 'T':
        is_fpi = True

    return is_complex, is_fpi


def list_datafiles():
    '''Get the type of the tomodir and the highest iteration to list all files,
    which will be plotted.
    '''
    is_cplx, is_fpi = td_type()
    # get the highest iteration
    it_rho = read_iter(is_fpi)
    it_phase = read_iter(False)
    # list the files
    files = ['inv/coverage.mag']
    dtype = ['cov']
    if isinstance(it_rho, str) or it_rho > 0:
        files.append(it_rho)
        dtype.append('mag')

    if (isinstance(it_rho, str) or it_rho > 0) and is_cplx:
        files.append(it_rho.replace('mag', 'pha'))
        dtype.append('pha')
    if (isinstance(it_phase, str) or it_phase > 0) and is_fpi:
        files.append(it_phase.replace('mag', 'pha'))
        dtype.append('pha_fpi')

    return files, dtype


def read_datafiles(files, dtype, column):
    '''Load the datafiles and return cov, mag, phase and fpi phase values.
    '''
    mag = []
    pha = []
    pha_fpi = []
    for filename, filetype in zip(files, dtype):
        if filetype == 'cov':
            cov = load_cov(filename)
        elif filetype == 'mag':
            mag = load_rho(filename, column)
        elif filetype == 'pha':
            pha = load_rho(filename, 2)
        elif filetype == 'pha_fpi':
            pha_fpi = load_rho(filename, 2)

    return cov, mag, pha, pha_fpi


def load_cov(name):
    '''Load a datafile with coverage file structure.
    '''
    if not os.path.isfile(name):
        return None
    # we need to support an older file format, therefore use the number of
    # columns in the header to detect the format
    header = np.loadtxt(name, max_rows=1)
    if header.size == 2:
        content = np.genfromtxt(
            name, skip_header=1, usecols=([2])
        )
    else:
        content = np.genfromtxt(
            name, skip_header=1, skip_footer=1, usecols=([2])
        )

    return content


def load_rho(name, column):
    '''Load a datafile with rho structure like mag and phase
    '''
    try:
        content = np.loadtxt(name, skiprows=1, usecols=([column]))
    except Exception:
        raise ValueError('Given column to open does not exist.')

    return content


def calc_complex(rmag, rpha):
    ''' Calculate real and imaginary part of the complex conductivity from
    magnitude and phase in log10.
    '''
    crho = 10 ** rmag * np.exp(1j * rpha / 1000.0)
    csigma = 1 / crho
    return csigma.real, csigma.imag


def plot_real(cid, ax, plotman, title, alpha, vmin, vmax,
              xmin, xmax, zmin, zmax, xunit, cbtiks, elecs):
    '''Plot real parts of the complex conductivity using the real_options.
    '''
    # handle options
    cblabel = units.get_label('log_real')
    zlabel = 'z [' + xunit + ']'
    xlabel = 'x [' + xunit + ']'
    cm = 'jet_r'
    xmin, xmax, zmin, zmax, vmin, vmax = check_minmax(
        plotman,
        cid,
        xmin, xmax,
        zmin, zmax,
        vmin, vmax,
    )
    # plot
    fig, ax, cnorm, cmap, cb, scalarMap = plotman.plot_elements_to_ax(
        cid=cid,
        cid_alpha=alpha,
        ax=ax,
        xmin=xmin,
        xmax=xmax,
        zmin=zmin,
        zmax=zmax,
        cblabel=cblabel,
        cbnrticks=cbtiks,
        title=title,
        zlabel=zlabel,
        xlabel=xlabel,
        plot_colorbar=True,
        cmap_name=cm,
        no_elecs=elecs,
        cbmin=vmin,
        cbmax=vmax,
    )
    return fig, ax, cnorm, cmap, cb


def plot_imag(cid, ax, plotman, title, alpha, vmin, vmax,
              xmin, xmax, zmin, zmax, xunit, cbtiks, elecs):
    '''Plot imag parts of the complex conductivity using the imag_options.
    '''
    # handle options
    cblabel = units.get_label('log_imag')
    zlabel = 'z [' + xunit + ']'
    xlabel = 'x [' + xunit + ']'
    cm = 'plasma_r'
    xmin, xmax, zmin, zmax, vmin, vmax = check_minmax(
        plotman,
        cid,
        xmin, xmax,
        zmin, zmax,
        vmin, vmax,
    )
    print('IMAG vmin/vmax', vmin, vmax)
    # plot
    fig, ax, cnorm, cmap, cb, scalarMap = plotman.plot_elements_to_ax(
        cid=cid,
        cid_alpha=alpha,
        ax=ax,
        xmin=xmin,
        xmax=xmax,
        zmin=zmin,
        zmax=zmax,
        cblabel=cblabel,
        cbnrticks=cbtiks,
        title=title,
        zlabel=zlabel,
        xlabel=xlabel,
        plot_colorbar=True,
        cmap_name=cm,
        no_elecs=elecs,
        cbmin=vmin,
        cbmax=vmax,
    )
    return fig, ax, cnorm, cmap, cb


def plot_mag(cid, ax, plotman, title, unit, alpha, vmin, vmax,
             xmin, xmax, zmin, zmax, xunit, cbtiks, elecs):
    '''Plot magnitude of the complex resistivity using the mag_options.
    '''
    # handle options
    cblabel = units.get_label(unit)
    zlabel = 'z [' + xunit + ']'
    xlabel = 'x [' + xunit + ']'
    xmin, xmax, zmin, zmax, vmin, vmax = check_minmax(
        plotman,
        cid,
        xmin, xmax,
        zmin, zmax,
        vmin, vmax,
    )
    # plot
    fig, ax, cnorm, cmap, cb, scalarMap = plotman.plot_elements_to_ax(
        cid=cid,
        ax=ax,
        cid_alpha=alpha,
        xmin=xmin,
        xmax=xmax,
        zmin=zmin,
        zmax=zmax,
        cblabel=cblabel,
        cbnrticks=cbtiks,
        title=title,
        zlabel=zlabel,
        xlabel=xlabel,
        plot_colorbar=True,
        no_elecs=elecs,
        cbmin=vmin,
        cbmax=vmax,
        cmap_name='jet_r',
    )
    return fig, ax, cnorm, cmap, cb


def plot_pha(cid, ax, plotman, title, alpha, vmin, vmax,
             xmin, xmax, zmin, zmax, xunit, cbtiks, elecs):
    '''Plot phase of the complex resistivity using the pha_options.
    '''
    # handle options
    cblabel = units.get_label('phi')
    zlabel = 'z [' + xunit + ']'
    xlabel = 'x [' + xunit + ']'
    cm = 'plasma'
    cm = 'jet_r'
    xmin, xmax, zmin, zmax, vmin, vmax = check_minmax(
        plotman,
        cid,
        xmin, xmax,
        zmin, zmax,
        vmin, vmax,
    )
    # plot
    fig, ax, cnorm, cmap, cb, scalarMap = plotman.plot_elements_to_ax(
        cid=cid,
        ax=ax,
        cid_alpha=alpha,
        xmin=xmin,
        xmax=xmax,
        zmin=zmin,
        zmax=zmax,
        cblabel=cblabel,
        cbnrticks=cbtiks,
        title=title,
        zlabel=zlabel,
        xlabel=xlabel,
        plot_colorbar=True,
        cmap_name=cm,
        no_elecs=elecs,
        cbmin=vmin,
        cbmax=vmax,
    )
    return fig, ax, cnorm, cmap, cb


def plot_cov(cid, ax, plotman, title, vmin, vmax,
             xmin, xmax, zmin, zmax, xunit, cbtiks, elecs):
    '''Plot coverage of the complex resistivity using the cov_options.
    '''
    # handle options
    cblabel = units.get_label('cov')
    zlabel = 'z [' + xunit + ']'
    xlabel = 'x [' + xunit + ']'
    cm = 'GnBu'
    xmin, xmax, zmin, zmax, vmin, vmax = check_minmax(
        plotman,
        cid,
        xmin, xmax,
        zmin, zmax,
        vmin, vmax,
    )
    # plot
    fig, ax, cnorm, cmap, cb, scalarMap = plotman.plot_elements_to_ax(
        cid=cid,
        ax=ax,
        xmin=xmin,
        xmax=xmax,
        zmin=zmin,
        zmax=zmax,
        cblabel=cblabel,
        cbnrticks=cbtiks,
        title=title,
        zlabel=zlabel,
        xlabel=xlabel,
        plot_colorbar=True,
        cmap_name=cm,
        no_elecs=elecs,
        cbmin=vmin,
        cbmax=vmax,
    )
    return fig, ax, cnorm, cmap, cb


def plot_ratio(cid, ax, plotman, title, alpha, vmin, vmax,
               xmin, xmax, zmin, zmax, xunit, cbtiks, elecs):
    '''Plot ratio of two conductivity directions.
    '''
    # handle options
    cblabel = 'anisotropy ratio'
    zlabel = 'z [' + xunit + ']'
    xlabel = 'x [' + xunit + ']'
    # cm = 'brg'
    cm = 'RdYlGn'
    xmin, xmax, zmin, zmax, vmin, vmax = check_minmax(
        plotman,
        cid,
        xmin, xmax,
        zmin, zmax,
        vmin, vmax,
    )
    # plot
    fig, ax, cnorm, cmap, cb, scalarMap = plotman.plot_elements_to_ax(
        cid=cid,
        ax=ax,
        xmin=xmin,
        xmax=xmax,
        zmin=zmin,
        zmax=zmax,
        cblabel=cblabel,
        cbnrticks=cbtiks,
        title=title,
        zlabel=zlabel,
        xlabel=xlabel,
        plot_colorbar=True,
        cmap_name=cm,
        no_elecs=elecs,
        cbmin=vmin,
        cbmax=vmax,
    )
    return fig, ax, cnorm, cmap, cb


def alpha_from_cov(plotman, alpha_cov):
    '''Calculate alpha values from the coverage/2.5.
    '''
    cov_file = 'inv/coverage.mag'
    if not os.path.isfile(cov_file):
        return None, plotman

    abscov = np.abs(load_cov(cov_file))

    if alpha_cov:
        normcov = np.divide(abscov, 2.5)
        normcov[np.where(normcov > 1)] = 1
        mask = np.subtract(1, normcov)
        alpha = plotman.parman.add_data(mask)
    else:
        alpha = plotman.parman.add_data(np.ones(len(abscov)))
    return alpha, plotman


def check_minmax(plotman, cid, xmin, xmax, zmin, zmax, vmin, vmax):
    """
    Get min and max values for axes and colorbar if not given
    """
    if xmin is None:
        xmin = plotman.grid.grid['x'].min()
    if xmax is None:
        xmax = plotman.grid.grid['x'].max()
    if zmin is None:
        zmin = plotman.grid.grid['z'].min()
    if zmax is None:
        zmax = plotman.grid.grid['z'].max()
    if isinstance(cid, int):
        subdata = plotman.parman.parsets[cid]
    else:
        subdata = cid
    if vmin is None:
        vmin = np.nanmin(subdata)
    if vmax is None:
        vmax = np.nanmax(subdata)

    return xmin, xmax, zmin, zmax, vmin, vmax


def getfigsize(plotman):
    '''calculate appropriate sizes for the subfigures
    '''
    xmin = plotman.grid.grid['x'].min()
    xmax = plotman.grid.grid['x'].max()
    zmin = plotman.grid.grid['z'].min()
    zmax = plotman.grid.grid['z'].max()
    if np.abs(zmax - zmin) < np.abs(xmax - xmin):
        sizex = 10 / 2.54
        sizez = 1.2 * sizex * (np.abs(zmax - zmin) / np.abs(xmax - xmin))
    else:
        sizez = 10 / 2.54
        sizex = sizez * (np.abs(xmax - xmin) / np.abs(zmax - zmin))
    # add 1 inch to accommodate colorbar
    sizex += 1.3
    sizez += 1
    return sizex, sizez


def create_non_dcplots(plotman, ax, mag, pha, options, alpha):
    if pha != []:
        cid = plotman.parman.add_data(pha)
        plot_pha(
            cid, ax[0, 1], plotman, 'Phase', alpha,
            options.pha_vmin, options.pha_vmax,
            options.xmin, options.xmax, options.zmin, options.zmax,
            options.unit, options.pha_cbtiks, options.no_elecs,
        )
        [real, imag] = calc_complex(mag, pha)
        cid_re = plotman.parman.add_data(real)
        cid_im = plotman.parman.add_data(np.log10(imag))
        plot_real(
            cid_re, ax[0, 2], plotman, 'Real Part', alpha,
            options.real_vmin, options.real_vmax,
            options.xmin, options.xmax, options.zmin, options.zmax,
            options.unit, options.real_cbtiks, options.no_elecs,
        )
        plot_imag(
            cid_im, ax[0, 3], plotman, 'Imaginary Part', alpha,
            options.imag_vmin, options.imag_vmax,
            options.xmin, options.xmax, options.zmin, options.zmax,
            options.unit, options.imag_cbtiks, options.no_elecs,
        )
    else:
        ax[0, 1].axis('off')
        ax[0, 2].axis('off')
        ax[0, 3].axis('off')


def create_fpiplots(plotman, ax, mag, pha_fpi, options, alpha):
    if pha_fpi != []:
        cid = plotman.parman.add_data(pha_fpi)
        plot_pha(cid, ax[1, 1], plotman, 'FPI Phase', alpha,
                 options.pha_vmin, options.pha_vmax,
                 options.xmin, options.xmax, options.zmin, options.zmax,
                 options.unit, options.pha_cbtiks, options.no_elecs,
                 )
        [real, imag] = calc_complex(mag, pha_fpi)
        cid_fre = plotman.parman.add_data(real)
        cid_fim = plotman.parman.add_data(imag)
        plot_real(cid_fre, ax[1, 2], plotman, 'FPI Real Part', alpha,
                  options.real_vmin, options.real_vmax,
                  options.xmin, options.xmax, options.zmin, options.zmax,
                  options.unit, options.real_cbtiks, options.no_elecs,
                  )
        plot_imag(cid_fim, ax[1, 3], plotman, 'FPI Imaginary Part',
                  alpha, options.imag_vmin, options.imag_vmax,
                  options.xmin, options.xmax, options.zmin, options.zmax,
                  options.unit, options.imag_cbtiks, options.no_elecs,
                  )
    else:
        ax[1, 1].axis('off')
        ax[1, 2].axis('off')
        ax[1, 3].axis('off')


def create_tdplot(plotman, cov, mag, pha, pha_fpi, alpha, options):
    '''Plot the data of the tomodir in one overview plot.
    '''
    sizex, sizez = getfigsize(plotman)
    # create figure
    f, ax = plt.subplots(2, 4, figsize=(4 * sizex, 2 * sizez))
    if options.title is not None:
        plt.suptitle(options.title, fontsize=18)
    # plot magnitue
    if options.cmaglin:
        cid = plotman.parman.add_data(np.power(10, mag))
        loglin = 'rho'
    else:
        cid = plotman.parman.add_data(mag)
        loglin = 'log_rho'
    plot_mag(cid, ax[0, 0], plotman, 'Magnitude', loglin, alpha,
             options.mag_vmin, options.mag_vmax,
             options.xmin, options.xmax, options.zmin, options.zmax,
             options.unit, options.mag_cbtiks, options.no_elecs,
             )
    # plot coverage
    cid = plotman.parman.add_data(cov)
    plot_cov(cid, ax[1, 0], plotman, 'Coverage',
             options.cov_vmin, options.cov_vmax,
             options.xmin, options.xmax, options.zmin, options.zmax,
             options.unit, options.cov_cbtiks, options.no_elecs,
             )
    # plot phase, real, imag
    create_non_dcplots(plotman, ax, mag, pha, options, alpha)
    # plot fpi phase, real, imag
    create_fpiplots(plotman, ax, mag, pha_fpi, options, alpha)
    f.tight_layout()
    f.savefig('td_overview.png', dpi=300)
    return f, ax


def create_singleplots(plotman, cov, mag, pha, pha_fpi, alpha, options):
    '''Plot the data of the tomodir in individual plots.
    '''
    if len(mag) == 0:
        mag = np.ones(plotman.grid.nr_of_elements) * np.nan
    if cov is None:
        cov = np.ones_like(mag) * np.nan

    magunit = 'log_rho'
    if not pha == []:
        [real, imag] = calc_complex(mag, pha)
        if options.c_in_log:
            real = np.log10(real)
            with np.errstate(divide='ignore'):
                imag = np.log10(imag)
            imag[np.isinf(imag)] = np.nan

        if not pha_fpi == []:
            [real_fpi, imag_fpi] = calc_complex(mag, pha_fpi)
            if options.cmaglin:
                mag = np.power(10, mag)
                magunit = 'rho'
            if options.c_in_log:
                real_fpi = np.log10(real_fpi)
                with np.errstate(divide='ignore'):
                    imag_fpi = np.log10(imag_fpi)
                imag_fpi[np.isinf(imag_fpi)] = np.nan
            # import IPython
            # IPython.embed()
            # print(imag_fpi, np.nanmin(imag_fpi), np.nanmax(imag_fpi))
            # print(options.imag_vmin, options.imag_vmax)
            # exit()

            data = np.column_stack((
                mag, cov, pha, real, imag,
                pha_fpi, real_fpi, imag_fpi
            ))
            titles = [
                'Magnitude', 'Coverage',
                'Phase', 'Real Part', 'Imaginary Part',
                'FPI Phase', 'FPI Real Part', 'FPI Imaginary Part'
            ]
            unites = [
                magunit, 'cov',
                'phi', 'log_real', 'log_imag',
                'phi', 'log_real', 'log_imag'
            ]
            vmins = [
                options.mag_vmin, options.cov_vmin,
                options.pha_vmin, options.real_vmin, options.imag_vmin,
                options.pha_vmin, options.real_vmin, options.imag_vmin
            ]
            vmaxs = [options.mag_vmax, options.cov_vmax,
                     options.pha_vmax, options.real_vmax, options.imag_vmax,
                     options.pha_vmax, options.real_vmax, options.imag_vmax]
            cmaps = ['jet', 'GnBu',
                     'jet_r', 'jet_r', 'plasma_r',
                     'jet_r', 'jet_r', 'plasma_r']
            saves = ['rho', 'cov',
                     'phi', 'real', 'imag',
                     'fpi_phi', 'fpi_real', 'fpi_imag']
        else:
            if options.cmaglin:
                mag = np.power(10, mag)
                magunit = 'rho'
            data = np.column_stack((mag, cov, pha, real, imag))
            titles = ['Magnitude', 'Coverage',
                      'Phase', 'Real Part', 'Imaginary Part']
            unites = [magunit, 'cov',
                      'phi', 'log_real', 'log_imag']
            vmins = [options.mag_vmin, options.cov_vmin,
                     options.pha_vmin, options.real_vmin, options.imag_vmin]
            vmaxs = [options.mag_vmax, options.cov_vmax,
                     options.pha_vmax, options.real_vmax, options.imag_vmax]
            cmaps = ['jet', 'GnBu',
                     'jet_r', 'jet_r', 'plasma_r']
            saves = ['rho', 'cov',
                     'phi', 'real', 'imag']
    else:
        data = np.column_stack((mag, cov))
        titles = ['Magnitude', 'Coverage']
        unites = [magunit, 'cov']
        vmins = [options.mag_vmin, options.cov_vmin]
        vmaxs = [options.mag_vmax, options.cov_vmax]
        cmaps = ['jet', 'GnBu']
        saves = ['rho', 'cov']
    try:
        mod_rho = np.genfromtxt('rho/rho.dat', skip_header=1, usecols=([0]))
        mod_pha = np.genfromtxt('rho/rho.dat', skip_header=1, usecols=([1]))
        if data.size == 0:
            data = np.column_stack((mod_rho, mod_pha))
        else:
            data = np.column_stack((data, mod_rho, mod_pha))
        titles.append('Model')
        titles.append('Model')
        unites.append('rho')
        unites.append('phi')
        vmins.append(options.mag_vmin)
        vmins.append(options.pha_vmin)
        vmaxs.append(options.mag_vmax)
        vmaxs.append(options.pha_vmax)
        cmaps.append('jet')
        cmaps.append('plasma')
        saves.append('rhomod')
        saves.append('phamod')
    except Exception:
        pass

    for datum, title, unit, vmin, vmax, cm, save in zip(
            np.transpose(data), titles, unites, vmins, vmaxs, cmaps, saves):
        if len(datum) == 0:
            continue
        if np.all(np.isnan(datum)):
            continue
        # print(save)
        # if save == 'fpi_imag':
        #     import IPython
        #     IPython.embed()
        if save == 'rho' and options.cmaglin:
            datum = np.power(10, datum)
            unit = 'rho'

        sizex, sizez = getfigsize(plotman)
        f, ax = plt.subplots(1, figsize=(sizex, sizez))
        cid = plotman.parman.add_data(datum)
        # handle options
        cblabel = units.get_label(unit)
        if options.title is not None:
            title = options.title
        zlabel = 'z [' + options.unit + ']'
        xlabel = 'x [' + options.unit + ']'
        xmin, xmax, zmin, zmax, vmin, vmax = check_minmax(
            plotman,
            cid,
            options.xmin, options.xmax,
            options.zmin, options.zmax,
            vmin, vmax
        )
        # plot
        cmap = mpl_cm.get_cmap(cm)
        fig, ax, cnorm, cmap, cb, scalarMap = plotman.plot_elements_to_ax(
                cid=cid,
                cid_alpha=alpha,
                ax=ax,
                xmin=xmin,
                xmax=xmax,
                zmin=zmin,
                zmax=zmax,
                cblabel=cblabel,
                title=title,
                zlabel=zlabel,
                xlabel=xlabel,
                plot_colorbar=True,
                cmap_name=cm,
                over=cmap(1.0),
                under=cmap(0.0),
                no_elecs=options.no_elecs,
                cbmin=vmin,
                cbmax=vmax,
                )
        f.tight_layout()
        f.savefig(save + '.png', dpi=300)


def create_anisomagplot(plotman, x, y, z, alpha, options):
    '''Plot the data of the tomodir in one overview plot.
    '''
    sizex, sizez = getfigsize(plotman)
    # create figure
    f, ax = plt.subplots(2, 3, figsize=(3 * sizex, 2 * sizez))
    if options.title is not None:
        plt.suptitle(options.title, fontsize=18)
        plt.subplots_adjust(wspace=1.5, top=2)
    # plot magnitue
    if options.cmaglin:
        cidx = plotman.parman.add_data(np.power(10, x))
        cidy = plotman.parman.add_data(np.power(10, y))
        cidz = plotman.parman.add_data(np.power(10, z))
        loglin = 'rho'
    else:
        cidx = plotman.parman.add_data(x)
        cidy = plotman.parman.add_data(y)
        cidz = plotman.parman.add_data(z)
        loglin = 'log_rho'
    cidxy = plotman.parman.add_data(np.divide(x, y))
    cidyz = plotman.parman.add_data(np.divide(y, z))
    cidzx = plotman.parman.add_data(np.divide(z, x))
    plot_mag(cidx, ax[0, 0], plotman, 'x', loglin, alpha,
             options.mag_vmin, options.mag_vmax,
             options.xmin, options.xmax, options.zmin, options.zmax,
             options.unit, options.mag_cbtiks, options.no_elecs,
             )
    plot_mag(cidy, ax[0, 1], plotman, 'y', loglin, alpha,
             options.mag_vmin, options.mag_vmax,
             options.xmin, options.xmax, options.zmin, options.zmax,
             options.unit, options.mag_cbtiks, options.no_elecs,
             )
    plot_mag(cidz, ax[0, 2], plotman, 'z', loglin, alpha,
             options.mag_vmin, options.mag_vmax,
             options.xmin, options.xmax, options.zmin, options.zmax,
             options.unit, options.mag_cbtiks, options.no_elecs,
             )
    plot_ratio(cidxy, ax[1, 0], plotman, 'x/y', alpha,
               options.rat_vmin, options.rat_vmax,
               options.xmin, options.xmax, options.zmin, options.zmax,
               options.unit, options.mag_cbtiks, options.no_elecs,
               )
    plot_ratio(cidyz, ax[1, 1], plotman, 'y/z', alpha,
               options.rat_vmin, options.rat_vmax,
               options.xmin, options.xmax, options.zmin, options.zmax,
               options.unit, options.mag_cbtiks, options.no_elecs,
               )
    plot_ratio(cidzx, ax[1, 2], plotman, 'z/x', alpha,
               options.rat_vmin, options.rat_vmax,
               options.xmin, options.xmax, options.zmin, options.zmax,
               options.unit, options.mag_cbtiks, options.no_elecs,
               )
    f.tight_layout()
    f.savefig('mag_aniso.png', dpi=300)
    return f, ax


def create_anisophaplot(plotman, x, y, z, alpha, options):
    '''Plot the data of the tomodir in one overview plot.
    '''
    sizex, sizez = getfigsize(plotman)
    # create figure
    f, ax = plt.subplots(2, 3, figsize=(3 * sizex, 2 * sizez))
    if options.title is not None:
        plt.suptitle(options.title, fontsize=18)
        plt.subplots_adjust(wspace=1, top=0.8)
    # plot phase
    cidx = plotman.parman.add_data(x)
    cidy = plotman.parman.add_data(y)
    cidz = plotman.parman.add_data(z)
    cidxy = plotman.parman.add_data(np.subtract(x, y))
    cidyz = plotman.parman.add_data(np.subtract(y, z))
    cidzx = plotman.parman.add_data(np.subtract(z, x))
    plot_pha(cidx, ax[0, 0], plotman, 'x', alpha,
             options.pha_vmin, options.pha_vmax,
             options.xmin, options.xmax, options.zmin, options.zmax,
             options.unit, options.pha_cbtiks, options.no_elecs,
             )
    plot_pha(cidy, ax[0, 1], plotman, 'y', alpha,
             options.pha_vmin, options.pha_vmax,
             options.xmin, options.xmax, options.zmin, options.zmax,
             options.unit, options.pha_cbtiks, options.no_elecs,
             )
    plot_pha(cidz, ax[0, 2], plotman, 'z', alpha,
             options.pha_vmin, options.pha_vmax,
             options.xmin, options.xmax, options.zmin, options.zmax,
             options.unit, options.pha_cbtiks, options.no_elecs,
             )
    plot_ratio(cidxy, ax[1, 0], plotman, 'x-y', alpha,
               options.rat_vmin, options.rat_vmax,
               options.xmin, options.xmax, options.zmin, options.zmax,
               options.unit, options.mag_cbtiks, options.no_elecs,
               )
    plot_ratio(cidyz, ax[1, 1], plotman, 'y-z', alpha,
               options.rat_vmin, options.rat_vmax,
               options.xmin, options.xmax, options.zmin, options.zmax,
               options.unit, options.mag_cbtiks, options.no_elecs,
               )
    plot_ratio(cidzx, ax[1, 2], plotman, 'z-x', alpha,
               options.rat_vmin, options.rat_vmax,
               options.xmin, options.xmax, options.zmin, options.zmax,
               options.unit, options.mag_cbtiks, options.no_elecs,
               )
    f.tight_layout()
    f.savefig('pha_aniso.png', dpi=300)
    return f, ax


def create_hlammagplot(plotman, h, ratio, alpha, options):
    '''Plot the data of the tomodir in one overview plot.
    '''
    sizex, sizez = getfigsize(plotman)
    # create figure
    f, ax = plt.subplots(1, 3, figsize=(3 * sizex, sizez))
    if options.title is not None:
        plt.suptitle(options.title, fontsize=18)
        plt.subplots_adjust(wspace=1, top=0.8)
    # plot magnitue
    if options.cmaglin:
        cidh = plotman.parman.add_data(np.power(10, h))
        cidv = plotman.parman.add_data(
                np.divide(np.power(10, h), np.power(10, ratio)))
        loglin = 'rho'
    else:
        cidh = plotman.parman.add_data(h)
        cidv = plotman.parman.add_data(
                np.log10(np.divide(np.power(10, h), np.power(10, ratio))))
        loglin = 'log_rho'

    cidr = plotman.parman.add_data(np.power(10, ratio))
    plot_mag(cidh, ax[0], plotman, 'horizontal', loglin, alpha,
             options.mag_vmin, options.mag_vmax,
             options.xmin, options.xmax, options.zmin, options.zmax,
             options.unit, options.mag_cbtiks, options.no_elecs,
             )
    plot_mag(cidv, ax[1], plotman, 'vertical', loglin, alpha,
             options.mag_vmin, options.mag_vmax,
             options.xmin, options.xmax, options.zmin, options.zmax,
             options.unit, options.mag_cbtiks, options.no_elecs,
             )
    plot_ratio(cidr, ax[2], plotman, 'hor/ver', alpha,
               options.rat_vmin, options.rat_vmax,
               options.xmin, options.xmax, options.zmin, options.zmax,
               options.unit, options.mag_cbtiks, options.no_elecs,
               )
    f.tight_layout()
    f.savefig('mag_hlam.png', dpi=300)
    return f, ax


def create_hlamphaplot(plotman, h, v, alpha, options):
    '''Plot the data of the tomodir in one overview plot.
    '''
    sizex, sizez = getfigsize(plotman)
    # create figure
    f, ax = plt.subplots(1, 3, figsize=(3 * sizex, sizez))
    if options.title is not None:
        plt.suptitle(options.title, fontsize=18)
        plt.subplots_adjust(wspace=1, top=0.8)
    cidh = plotman.parman.add_data(h)
    cidv = plotman.parman.add_data(v)

    cidr = plotman.parman.add_data(np.subtract(h, v))
    plot_pha(cidh, ax[0], plotman, 'horizontal', alpha,
             options.pha_vmin, options.pha_vmax,
             options.xmin, options.xmax, options.zmin, options.zmax,
             options.unit, options.pha_cbtiks, options.no_elecs,
             )
    plot_pha(cidv, ax[1], plotman, 'vertical', alpha,
             options.pha_vmin, options.pha_vmax,
             options.xmin, options.xmax, options.zmin, options.zmax,
             options.unit, options.pha_cbtiks, options.no_elecs,
             )
    plot_ratio(cidr, ax[2], plotman, 'hor - ver', alpha,
               options.rat_vmin, options.rat_vmax,
               options.xmin, options.xmax, options.zmin, options.zmax,
               options.unit, options.pha_cbtiks, options.no_elecs,
               )
    f.tight_layout()
    f.savefig('pha_hlam.png', dpi=300)
    return f, ax


def main():
    if os.path.basename(os.getcwd()) == 'exe':
        os.chdir('..')

    options = handle_options()
    matplotlib.style.use('default')
    mpl_style.general_settings()
    # load grid
    grid = CRGrid.crt_grid('grid/elem.dat',
                           'grid/elec.dat')
    plotman = CRPlot.plotManager(grid=grid)
    # get alpha
    alpha, plotman = alpha_from_cov(plotman, options.alpha_cov)
    # make tomodir overview plot
    if not options.single and not options.aniso and not options.hlam:
        [datafiles, filetype] = list_datafiles()
        [cov, mag, pha, pha_fpi] = read_datafiles(
                datafiles,
                filetype,
                options.column)
        create_tdplot(plotman, cov, mag, pha, pha_fpi, alpha, options)
    # make individual plots
    elif options.single and not options.aniso and not options.hlam:
        [datafiles, filetype] = list_datafiles()
        [cov, mag, pha, pha_fpi] = read_datafiles(
                datafiles,
                filetype,
                options.column)
        create_singleplots(plotman, cov, mag, pha, pha_fpi, alpha, options)
    # make plots of anisotropic results
    elif options.aniso and not options.single and not options.hlam:
        filename = read_iter(False)
        x = load_rho(filename, 2)
        y = load_rho(filename, 3)
        z = load_rho(filename, 4)
        create_anisomagplot(plotman, x, y, z, alpha, options)
        x = load_rho(filename[:-3] + 'pha', 2)
        y = load_rho(filename[:-3] + 'pha', 3)
        z = load_rho(filename[:-3] + 'pha', 4)
        create_anisophaplot(plotman, x, y, z, alpha, options)
    elif options.hlam and not options.single and not options.aniso:
        filename = read_iter(False)
        hor = load_rho(filename, 2)
        lam = load_rho(filename, 3)
        create_hlammagplot(plotman, hor, lam, alpha, options)
        hor = load_rho(filename[:-3] + 'pha', 2)
        ver = load_rho(filename[:-3] + 'pha', 4)
        create_hlamphaplot(plotman, hor, ver, alpha, options)
    else:
        print(
            'You can only use one option out of these: '
            '"single", "hlam" or "aniso", not two at the same time.'
        )
        exit()


if __name__ == '__main__':
    main()



# *-* coding: utf-8 *-*
"""Manage measurement configurations and corresponding measurements.

Geometric Factors
-----------------

.. plot:: pyplots/plot_K_vs_dipol_sep.py

Literature
----------

* Noel, Mark and Biwen Xu; Archaeological investigation by electrical
    resistivity tomography: a preliminary study. Geophys J Int 1991; 107 (1):
    95-102. doi: 10.1111/j.1365-246X.1991.tb01159.x
* Stummer, Peter, Hansruedi Maurer, and Alan G. Green. “Experimental Design:
  Electrical Resistivity Data Sets That Provide Optimum Subsurface
  Information.” Geophysics 69, no. 1 (January 1, 2004): 120–120.
  doi:10.1190/1.1649381.
* Roy, A., and A. Apparao. “DEPTH OF INVESTIGATION IN DIRECT CURRENT METHODS.”
  GEOPHYSICS 36, no. 5 (January 1, 1971): 943–59. doi:10.1190/1.1440226.

"""
import numpy as np
import pandas as pd

import crtomo.mpl

import reda.configs.configManager as reda_config_mgr
plt, mpl = crtomo.mpl.setup()


class ConfigManager(reda_config_mgr.ConfigManager):
    def __init__(self, **kwargs):
        """

        nr_of_electrodes : int
            Number of electrodes
        """
        # assert 'nr_of_electrodes' in kwargs
        super().__init__(**kwargs)
        # store measurements in a list of size N arrays
        self.measurements = {}
        # global counter for measurements
        self.meas_counter = - 1

    def clear_measurements(self):
        """Remove all measurements from self.measurements. Reset the
        measurement counter. All ID are invalidated.
        """
        keys = list(self.measurements.keys())
        for key in keys:
            del(self.measurements[key])
        self.meas_counter = -1

    def delete_measurements(self, mid):
        del(self.measurements[mid])

    def add_measurements(self, measurements):
        """Add new measurements to this instance

        Parameters
        ----------
        measurements: numpy.ndarray
            one or more measurement sets. It must either be 1D or 2D, with the
            first dimension the number of measurement sets (K), and the second
            the number of measurements (N): K x N

        Returns
        -------
        mid: int
            measurement ID used to extract the measurements later on

        Examples
        --------
        >>> import numpy as np
            import crtomo.configManager as CRconfig
            config = CRconfig.ConfigManager(nr_of_electrodes=10)
            config.gen_dipole_dipole(skipc=0)
            # generate some random noise
            random_measurements = np.random.random(config.nr_of_configs)
            mid = config.add_measurements(random_measurements)
            # retrieve using mid
            print(config.measurements[mid])

        """
        subdata = np.atleast_2d(measurements)

        if self.configs is None:
            raise Exception(
                'must read in configuration before measurements can be stored'
            )

        # we try to accommodate transposed input
        if subdata.shape[1] != self.configs.shape[0]:
            if subdata.shape[0] == self.configs.shape[0]:
                subdata = subdata.T
            else:
                raise Exception(
                    'Number of measurements does not match number of configs'
                )

        return_ids = []
        for dataset in subdata:
            cid = self._get_next_index()
            self.measurements[cid] = dataset.copy()
            return_ids.append(cid)

        if len(return_ids) == 1:
            return return_ids[0]
        else:
            return return_ids

    def remove_using_nr_injections(self, min_nr_of_injections):
        """Remove all measurements with a current injection that is not used a
        minimum number of times.

        This is useful to optimize measurement time for multi-channel systems,
        such as the IRIS Instruments Syscal Pro. The device can theoretically
        only facilitate the full number of channels if a corresponding number
        of voltage measurements is requested for a given current injection. As
        such, it can be useful to remove measurements with unique current
        injection dipoles. Note that other factors determine if all channels
        are actually used. Please refer to the device manual for
        device-specific information.

        Parameters
        ----------
        min_nr_of_injections : int
            Minimum number a given current injection should have to keep it

        """
        if min_nr_of_injections <= 0:
            return

        df = pd.DataFrame(self.configs)
        # group over A, B
        g = df.groupby([0, 1])
        df_filtered = g.filter(lambda x: x.shape[0] >= min_nr_of_injections)
        self.configs = df_filtered.values

    def plot_error_pars(self, mid):
        """ ??? DEFUNCT

        """
        R = None
        fig, axes = plt.subplots(1, 2, figsize=(10, 6))

        def plot_error_pars(axes, a, b, R, label=''):
            dR = a * R + b
            dlogR = np.abs(a + b / R)
            ax = axes[0]
            ax.scatter(R, dR / R * 100, label=label)
            ax = axes[1]
            ax.scatter(R, dlogR / np.abs(np.log(R)) * 100, label=label)
            ax.set_xscale('log')
            ax.set_yscale('log')

        for b in np.linspace(R.min(), np.percentile(R, 10), 5):
            plot_error_pars(axes, 0.05, b, R, label='b={0:.4f}'.format(b))
        axes[0].set_xlabel(r'$R [\Omega]$')
        axes[0].set_ylabel(r'$(\Delta R/R) \cdot 100 [\%]$')
        axes[1].set_xlabel(r'$log_{10}(R [\Omega])$')
        axes[1].set_ylabel(
            r'$log_{10}(R) / \Delta log_{10}(R) \cdot 100 [\%]$'
        )
        axes[0].axhline(y=100)
        axes[1].axhline(y=100)
        fig.tight_layout()
        fig.subplots_adjust(bottom=0.2)
        axes[0].legend(
            loc="lower center",
            ncol=4,
            bbox_to_anchor=(0, 0, 1, 1),
            bbox_transform=fig.transFigure
        )

        # fig.savefig('out.png', dpi=300)
        return fig

    def gen_voltage_dipoles_skip(self, cinj, skip=0):
        """For given current injections, generate all possible voltage dipoles
        with skip **skip**.

        Parameters
        ----------
        cinj : :py:class:`numpy.ndarray`
            Nx2 array containing the current injection electrodes a,b
        skip : int, optional
            Skip used for voltage electrodes. Default: 0

        """
        new_configs = []
        for ab in cinj:
            for i in range(1, self.nr_electrodes + 1):
                m = i
                n = i + skip + 1
                if m in ab or n in ab:
                    continue
                if n > self.nr_electrodes:
                    continue
                new_configs.append((ab[0], ab[1], m, n))
        configs = np.array(new_configs)
        self.add_to_configs(configs)
        return configs

    def load_crmod_data(self, data_source, is_forward_response=False,
                        try_fix_signs=False):
        """Load CRMod measurement data, either from file or directly from a
        numpy array.

        Parameters
        ----------
        data_source : string|numpy.ndarray
            if this is a string, treat it as an input filename. If it is a Nx5
            or Nx3 numpy array, use this data directly
        is_forward_response : bool, optional
            If True this indicated volt.dat file created by CRTomo during an
            inversion run to store forward responses of a given model
            iteration. In this case the third data column indicates the wdfak
            parameter, i.e., it indicates if a given data point was excluded
            from this iteration.
        try_fix_signs : bool, optional
            If True, try to fix sign reversals with respect to an already
            registered configuration set by swappend m and n entries.
        Returns
        -------
        cid_mag : int
            Measurement id for magnitude data
        cid_pha : int
            Measurement id for phase data
        mid_wdfak : int, optional
            Measurement id for wdfak indicators. Only returned if
            is_forward_response is True
        """
        if isinstance(data_source, str):
            with open(data_source, 'r') as fid:
                nr_of_configs = int(fid.readline().strip())
                measurements = np.loadtxt(fid)
                if nr_of_configs != measurements.shape[0]:
                    raise Exception(
                        'Indicated number of measurements is not equal '
                        'to actual number of measurements'
                    )
        elif isinstance(data_source, pd.DataFrame):
            measurements = data_source[
                ['a', 'b', 'm', 'n', 'r', 'rpha']
            ].values
        else:
            # assume numpy array
            # data already stored in the variable
            measurements = data_source

        if is_forward_response:
            # crmod ab-mn scheme
            abmn = self._crmod_to_abmn(measurements[:, 0:2])
            rmag = measurements[:, 2]
            if measurements.shape[1] == 4:
                # assume DC inversion
                rpha = None
                wdfak = measurements[:, 3]
            else:
                rpha = measurements[:, 3]
                wdfak = measurements[:, 4]
        else:
            assert measurements.shape[1] in (4, 6), \
                "Only know how to deal with 4 or 6 columns. " + \
                "DC probably not supported"

            if measurements.shape[1] == 4:
                # crmod ab-mn scheme
                abmn = self._crmod_to_abmn(measurements[:, 0:2])
                rmag = measurements[:, 2]
                rpha = measurements[:, 3]
            else:
                # abmn in separate columns
                abmn = measurements[:, 0:4]
                rmag = measurements[:, 4]
                rpha = measurements[:, 5]

        if self.configs is None:
            self.configs = abmn
        else:
            if try_fix_signs:
                if not np.all(abmn[:, 0:2] == self.configs[:, 0:2]):
                    raise Exception(
                        'try_fix_signs failed: a and b columns do not match'
                    )
                # find swapped potential electrodes
                indices_to_switch = np.where(
                    np.all(
                        abmn[:, 3:1:-1] == self.configs[:, 2:4], axis=1
                    )
                )
                # fix configurations
                abmn[indices_to_switch] = self.configs[indices_to_switch]
                # Fix impedances
                if rpha is not None:
                    z_complex = rmag[
                        indices_to_switch
                    ] * np.exp(1j * rpha[indices_to_switch] / 1000)
                    z_complex_fixed = z_complex * -1
                    rmag[indices_to_switch] = np.abs(z_complex_fixed)
                    rpha[indices_to_switch] = np.arctan2(
                        np.imag(z_complex_fixed), np.real(z_complex_fixed)
                    )
                else:
                    # DC case
                    print('WARNING: try_fix_signs with DC not validated!')
                    rmag[indices_to_switch] = -rmag[indices_to_switch]

            # check that configs match
            if not np.all(abmn == self.configs):
                raise Exception(
                    'previously stored configurations do not match new '
                    'configurations'
                )
        # add data
        cid_mag = self.add_measurements(rmag)
        if rpha is not None:
            cid_pha = self.add_measurements(rpha)
        else:
            cid_pha = None

        if is_forward_response:
            mid_wdfak = self.add_measurements(wdfak)
            return cid_mag, cid_pha, mid_wdfak

        return cid_mag, cid_pha

    def load_crmod_volt(self, filename):
        """Load a CRMod measurement file (commonly called volt.dat)

        Parameters
        ----------
        filename : string
            path to input filename

        Returns
        -------
        cid_mag : int
            Measurement id for magnitude data
        cid_pha : int
            Measurement id for phase data
        """
        cid_mag, cid_pha = self.load_crmod_data(filename)
        return cid_mag, cid_pha

    def delete_data_points(self, indices):
        """Delete data points by index (0-indexed), both in configs and
        measurements. Deletions will be done in ALL registered measurements to
        ensure consistency.

        Parameters
        ----------
        indices : int|iterable
            Indices to delete
        """
        print('Deleting configurations:')
        print(self.configs[indices])

        # first the configurations
        self.configs = np.delete(self.configs, indices, axis=0)

        for key in sorted(self.measurements.keys()):
            self.measurements[key] = np.delete(
                self.measurements[key], indices, axis=0
            )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
'''
init_sim.py
initialize a new simulation directory structure using `pwd` as the root
directory.
If there are files present in this directory defined in the directory
structure, they will me moved to the corresponding directories
command -s: if this parameter reads "silent", don't print any warnings.
'''
import os
import shutil
from optparse import OptionParser

import crtomo.cfg


def handle_cmd_options():
    '''
    Get  the options from the command line.
    '''
    parser = OptionParser()
    parser.add_option("-s", "--silent", action="store_true", dest="silent",
                      help="print any warnings", default=False)
    (options, args) = parser.parse_args()
    return options, args


def move(fname, folder, options):
    """Move file to dir if existing
    """
    if os.path.isfile(fname):
        shutil.move(fname, folder)
    else:
        if options.silent is False:
            print('{0} missing'.format(fname))


def main():
    (options, args) = handle_cmd_options()
    for i in ['config', 'exe', 'grid', 'mod', 'mod/sens', 'mod/pot',
              'inv', 'rho']:
        os.mkdir(i)
    files = ['config.dat', 'elem.dat', 'elec.dat', 'rho.dat', 'volt.dat',
             'crt.noisemod', 'decoupling.dat']
    folders = ['config', 'grid', 'grid', 'rho', 'mod', 'exe', 'exe']

    for i, j in zip(files, folders):
        move(i, j, options)

    if os.path.isfile('crmod.cfg'):
        shutil.move('crmod.cfg', 'exe')
    elif os.path.isfile('config/config.dat'):
        # only copy the crmod.cfg file if a config.dat file exists
        cfg = crtomo.cfg.crmod_config()
        cfg.write_to_file('exe/crmod.cfg')

    if os.path.isfile('crtomo.cfg'):
        shutil.move('crtomo.cfg', 'exe')
    else:
        cfg = crtomo.cfg.crtomo_config()
        cfg.write_to_file('exe/crtomo.cfg')


if __name__ == '__main__':
    main()



#!/usr/bin/env python
"""
Create homogenized version of a CRTomo grid.

Examples
--------


"""
import os
import shutil
from optparse import OptionParser
import subprocess
import numpy as np

from crtomo.mpl_setup import *


class grid_container():

    def __init__(self, electrodes_file=None, boundaries_file=None,
                 char_length_file=None):

        if char_length_file is not None and os.path.isfile(char_length_file):
            self.char_length_file = char_length_file
        else:
            self.char_length_file = None

        if electrodes_file is not None:
            self.electrodes = np.loadtxt(electrodes_file)
        else:
            self.electrodes = None

        if boundaries_file is not None:
            self.boundaries = np.loadtxt(boundaries_file)
        else:
            self.boundaries = None

        self.script = None

    def save_to_dir(self, directory):

        if not os.path.isdir(directory):
            os.makedirs(directory)
        pwd = os.getcwd()
        os.chdir(directory)

        np.savetxt('electrodes.dat', self.electrodes)
        np.savetxt('boundaries.dat', self.boundaries)
        if self.char_length_file is not None:
            shutil.copy(self.char_length_file, './')

        if self.script is not None:
            with open('create_map_grid.sh', 'w') as fid:
                fid.write(self.script)
            print('Calling create_map_grid.sh')
            subprocess.call('sh create_map_grid.sh', shell=True)
        os.chdir(pwd)


def handle_cmd_options():
    parser = OptionParser()
    parser.add_option(
        '-d', "--data_dir",
        dest="data_dir",
        type="string",
        help="data dir containing the file electrodes.dat " +
        "and boundaries.dat' (default: data/)",
        default="data",
    )
    parser.add_option(
        "-o", "--output", dest="output",
        help="Output directory (default: grid_hom)",
        metavar="DIR",
        default="grid_hom",
    )
    parser.add_option(
        "--dx", dest="dx",
        help="dx",
        type="float",
        metavar="FLOAT",
        default=50,
    )
    parser.add_option(
        "--dy", dest="dy",
        help="Depth of grid below electrodes (default: 100 m)",
        type="float",
        metavar="FLOAT",
        default=100,
    )

    (options, args) = parser.parse_args()
    return options


def rotate_point(xorigin, yorigin, x, y, angle):
    """Rotate the given point by angle
    """
    rotx = (x - xorigin) * np.cos(angle) - (y - yorigin) * np.sin(angle)
    roty = (x - yorigin) * np.sin(angle) + (y - yorigin) * np.cos(angle)
    return rotx, roty


def homogenize_grid(grid_old, dx, dy):
    """
    1) fit line through electrodes
    2) rotate electrodes so that line lies in the horizontal plane
    3) translate z-coordinates so that all z-coordinates are negative
    """

    # 1 line fit
    x = grid_old.electrodes[:, 0]
    y = grid_old.electrodes[:, 1]
    sort_indices = np.argsort(x)
    x_sort = x[sort_indices]
    y_sort = y[sort_indices]
    p = np.polyfit(x_sort, y_sort, 1)

    # 2. rotate around first electrode
    offsetx = x_sort[0]
    offsety = y_sort[0]

    alpha = -np.arctan2(p[0], 1.0)  # * 180 / np.pi

    xn = []
    yn = []
    for xc, yc in zip(x, y):
        rotx, roty = rotate_point(offsetx, offsety, xc, yc, alpha)
        xn.append(rotx + offsetx)
        yn.append(roty + offsety)

    new_coordinates = np.vstack((xn, yn)).T

    # move vertically

    # this line is a horizontal line
    p_rot = np.polyfit(
        new_coordinates[:, 0],
        new_coordinates[:, 1],
        1
    )
    y_rot = np.polyval(
        p_rot,
        new_coordinates[:, 0],
    )
    ymax = y_rot[0]

    new_coordinates_trans = np.copy(new_coordinates)
    new_coordinates_trans[:, 1] -= ymax

    #
    fig, ax = plt.subplots(1, 1)
    ax.scatter(x, y, color='r', label='original')

    ax.plot(
        x,
        np.polyval(p, x),
        '-',
        label='fit',
        color='r',
    )

    ax.scatter(
        xn, yn,
        color='c',
        label='rotated',
    )

    ax.plot(
        xn,
        y_rot,
        '-',
        label='fit',
        color='c',
    )

    ax.scatter(
        new_coordinates_trans[:, 0],
        new_coordinates_trans[:, 1],
        label='homog',
    )

    # plot the line through the new coordintes
    pnew = np.polyfit(
        new_coordinates_trans[:, 0],
        new_coordinates_trans[:, 1],
        1
    )
    ax.plot(
        new_coordinates_trans[:, 0],
        np.polyval(pnew, new_coordinates_trans[:, 0]),
        '-',
        label='fit homogenized',
    )

    ax.legend(loc='best')

    ax.set_xlabel('x [m]')
    ax.set_ylabel('y [m]')

    fig.tight_layout()
    fig.savefig('output_electrodes.png', dpi=300)

    # electrodes = np.hstack((
    # boundaries
    bx = new_coordinates_trans[sort_indices, 0]
    by = new_coordinates_trans[sort_indices, 1]
    btype = [12 for i in bx]

    # add boundary
    # get deepest boundary coordinate
    y1 = by[-1] - dy
    y2 = by[0] - dy

    ymin = min(y1, y2)

    bx = np.hstack(
        (bx[0] - dx,
         bx,
         [bx[-1] + dx, bx[-1] + dx, bx[0] - dx])
    )

    by = np.hstack(
        (by[0],
         by,
         [by[-1], ymin, ymin]
         )
    )

    btype = np.hstack((12,
                       btype,
                       [11, 11, 11]))

    boundaries = np.vstack((bx, by, btype)).T

    # fig, ax = plt.subplots(1, 1)
    # ax.scatter(bx, by)
    # ax.set_aspect('equal')
    # fig.tight_layout()
    # fig.savefig('boundaries.png', dpi=300)

    grid_new = grid_container(None, None, grid_old.char_length_file)
    grid_new.boundaries = np.copy(boundaries)
    grid_new.electrodes = np.copy(new_coordinates_trans)

    fig, ax = plt.subplots(1, 1)
    ax.scatter(bx, by, color='g', label='boundaries')
    ax.scatter(new_coordinates[:, 0], new_coordinates[:, 1], color='g',
               label='electrodes')
    ax.set_aspect('equal')
    ax.legend()
    fig.tight_layout()
    fig.savefig('output_boundaries.png', dpi=300)

    def transform_back(data):
        new_data = np.copy(data)
        new_data[:, 1] += ymax
        new_data[:, 0] -= offsetx
        new_data[:, 1] -= offsety

        tmpx = (new_data[:, 0]) * np.cos(-alpha) - (new_data[:, 1]) * np.sin(
            -alpha)
        tmpy = (new_data[:, 0]) * np.sin(-alpha) + (new_data[:, 1]) * np.cos(
            -alpha)
        tmpx += offsetx
        tmpy += offsety
        return tmpx, tmpy

    shell_script = ''
    shell_script += '#!/bin/bash\n'
    shell_script += 'cr_trig_create grid\n'

    cmd1 = ''.join((
        'grid_translate -e grid/elem.dat ',
        '--dx {0} --dz {1} -o elem_trans1.dat'.format(
            offsetx, ymax - offsety)
    ))
    cmd2 = ''.join((
        'grid_rotate -e elem_trans1.dat ',
        '-a {0} -o elem_trans1_rot1.dat'.format(-alpha * 180 / np.pi)
    ))
    cmd3 = ''.join((
        'grid_translate -e elem_trans1_rot1.dat ',
        '--dx {0} --dz {1} -o elem_trans1_rot1_trans2.dat'.format(
            offsetx, offsety)
    ))
    shell_script += cmd1 + '\n'
    shell_script += cmd2 + '\n'
    shell_script += cmd3 + '\n'

    shell_script += ''.join((
        'grid_plot_wireframe --fancy -t grid/elec.dat ',
        '-e elem_trans1_rot1_trans2.dat -o trans1_rot1_trans2.png'
    ))

    grid_new.script = shell_script

    tmpx, tmpy = transform_back(grid_new.electrodes)
    bx, by = transform_back(grid_new.boundaries[:, 0:2])

    grid_map = grid_container(char_length_file=grid_old.char_length_file)
    grid_map.electrodes = np.vstack((tmpx, tmpy)).T
    grid_map.boundaries = np.vstack((bx, by, grid_new.boundaries[:, 2])).T

    fig, ax = plt.subplots(1, 1)
    ax.scatter(tmpx, tmpy, color='r', label='new')
    ax.scatter(x, y, color='b', label='old')
    ax.scatter(bx, by, color='g', label='boundaries')
    ax.set_aspect('equal')
    ax.legend()
    fig.tight_layout()
    fig.savefig('output_map.png', dpi=300)

    return grid_new, grid_map


def main():
    options = handle_cmd_options()
    electrodes_file = options.data_dir + os.sep + 'electrodes.dat'
    boundaries_file = options.data_dir + os.sep + 'boundaries.dat'
    char_length_file = os.path.abspath(os.path.normpath(
        options.data_dir + os.sep + 'char_length.dat'
    ))

    for filename in (electrodes_file, boundaries_file):
        if not os.path.isfile(filename):
            raise Exception('Could find file: {0}'.format(filename))

    grid_old = grid_container(
        electrodes_file,
        boundaries_file,
        char_length_file,
    )
    grid_new, grid_map = homogenize_grid(grid_old, options.dx, options.dy)

    grid_new.save_to_dir(options.output)


if __name__ == '__main__':
    main()



#!/usr/bin/env python
"""
Plot a histogram of a given volt.dat file (mod/volt.dat).

If a command line parameter is provided, this file we be used for plotting.

Examples
--------

# volt_histogram volt_01_f_0.000.crt
loading file: volt_01_f_0.000.crt
minimum/maximum magnitude: 0.000399 / 4.500319 Ohm
minimum/maximum phase: -49.41 / 18.765 mrad
Magnitude percentile 10%: 0.0022736 (log10: -2.643285939416605)
Magnitude percentile 20%: 0.0041868 (log10: -2.3781177845042643)
Magnitude percentile 30%: 0.0072387 (log10: -2.1403394218581333)
Magnitude percentile 40%: 0.012341 (log10: -1.9086496476864212)
Magnitude percentile 50%: 0.0221025 (log10: -1.6555586007583747)
Magnitude percentile 60%: 0.0407628 (log10: -1.3897359919182128)
Magnitude percentile 70%: 0.089891 (log10: -1.0462837882000315)
Magnitude percentile 80%: 0.214467 (log10: -0.6686403331535711)
Magnitude percentile 90%: 0.6953 (log10: -0.15782764569149582)
Phase percentile 10%: -7.8405
Phase percentile 20%: -4.047
Phase percentile 30%: -2.715
Phase percentile 40%: -1.8599999999999999
Phase percentile 50%: -1.335
Phase percentile 60%: -0.66
Phase percentile 70%: 0.255
Phase percentile 80%: 1.4160000000000013
Phase percentile 90%: 4.132500000000004

# volt_histogram
loading file: mod/volt.dat
minimum/maximum magnitude: 0.000164396115 / 9.16193485 Ohm
minimum/maximum phase: 0.0 / 0.0 mrad
Magnitude percentile 10%: 0.00116352 (log10: -2.9342247593548754)
Magnitude percentile 20%: 0.00292693 (log10: -2.533587969324661)
Magnitude percentile 30%: 0.00668467 (log10: -2.1749200553584274)
Magnitude percentile 40%: 0.0135107 (log10: -1.869322193615539)
Magnitude percentile 50%: 0.0261565 (log10: -1.5824204939114044)
Magnitude percentile 60%: 0.048195 (log10: -1.3169977466790868)
Magnitude percentile 70%: 0.0959452 (log10: -1.0179765941996923)
Magnitude percentile 80%: 0.209225 (log10: -0.679386765549746)
Magnitude percentile 90%: 0.592489 (log10: -0.22731998487816202)
Phase percentile 10%: 0.0
Phase percentile 20%: 0.0
Phase percentile 30%: 0.0
Phase percentile 40%: 0.0
Phase percentile 50%: 0.0
Phase percentile 60%: 0.0
Phase percentile 70%: 0.0
Phase percentile 80%: 0.0
Phase percentile 90%: 0.0

"""
import sys
import os
import numpy as np

import crtomo.mpl
plt, mpl = crtomo.mpl.setup()


def main():

    # we check for these files
    voltfiles = [
        'mod/volt.dat',
        'volt.dat',
        '../mod/volt.dat',
    ]
    if len(sys.argv) > 1:
        voltfiles = [sys.argv[-1], ]

    # quit on the first hit
    volt_data = None
    for filename in voltfiles:
        if not os.path.isfile(filename):
            continue

        # read in number of measurements
        nr_measurements = np.genfromtxt(
            filename,
            usecols=[0, ], max_rows=1
        ).take(0).astype(int)

        print('loading file: {0}'.format(filename))
        volt_data = np.loadtxt(filename, skiprows=1, max_rows=nr_measurements)
        break

    if volt_data is None:
        raise Exception('no suitable volt.dat file found!')

    print('minimum/maximum magnitude: {0} / {1} Ohm'.format(
        np.min(volt_data[:, 2]),
        np.max(volt_data[:, 2]),
    ))
    print('minimum/maximum phase: {0} / {1} mrad'.format(
        np.min(volt_data[:, 3]),
        np.max(volt_data[:, 3]),
    ))

    for i in range(10, 100, 10):
        print('Magnitude percentile {0}%: {1:0.6} (log10: {2})'.format(
            i,
            np.percentile(volt_data[:, 2], i),
            np.log10(np.percentile(volt_data[:, 2], i)),
            # int((np.log10(a) / np.abs(np.log10(a))) *\
            # np.ceil(np.abs(np.log10(a))))
        ))

    for i in range(10, 100, 10):
        print('Phase percentile {0}%: {1}'.format(
            i,
            np.percentile(volt_data[:, 3], i),
        ))

    fig, axes = plt.subplots(2, 2, figsize=(7, 5))

    ax = axes[0, 0]
    ax.hist(volt_data[:, 2], 100)
    ax.set_xlabel(r'$|Z|~[\Omega]$')
    ax.set_ylabel(r'$\#$')

    ax = axes[0, 1]
    ax.hist(volt_data[:, 3], 100)
    ax.set_xlabel(r'$\phi~[mrad]$')
    ax.set_ylabel(r'$\#$')

    ax = axes[1, 0]
    ax.hist(np.log10(volt_data[:, 2]), 100)
    ax.set_xlabel(r'$log_{10}(|Z|~[\Omega])$')
    ax.set_ylabel(r'$\#$')
    ax.xaxis.set_major_locator(mpl.ticker.MaxNLocator(5))

    fig.tight_layout()

    axes[1, 1].set_visible(False)
    fig.savefig('volt_histogram.png', dpi=300)


if __name__ == '__main__':
    main()



# *-* coding: utf-8 *-*
"""Manage parameter sets for a given grid. A parameter set is a set of values
that correspond to the elements of the grid. Usually this is a resistivity or
phase distribution, but can also be a cumulated sensitivity distribution.
"""
import os
from collections.abc import Iterable

from scipy.stats import multivariate_normal
import scipy.interpolate as spi
import numpy as np
import shapely.geometry as shapgeo


class ParMan(object):
    """manage one or more parameter sets for a given
    :class:`crtomo.grid.crt_grid` object
    """
    def __init__(self, grid_obj):
        """
        Parameters
        ----------
        grid_obj: crtomo.grid.crt_grid
            The FE grid that the parameter sets refer to
        """
        # check if grid is already initialized
        if grid_obj.grid is None:
            raise Exception('Grid object is not initialized!')
        self.grid = grid_obj
        # we store the parameter sets in here
        self.parsets = {}
        self.metadata = {}
        # we assign indices to each data set stored in the manager. This index
        # should be unique over the life time of each instance. Therefore we
        # increase the counter for each added data set. This way we also
        # prevent conflicts when data sets are removed.
        self.index = -1

    def _get_next_index(self):
        self.index += 1
        return self.index

    def reset(self):
        """Resets the ParMan instance. This process deletes all data and
        metadata, and resets the index variable
        """
        self.index = -1
        del(self.parsets)
        self.parsets = {}
        del(self.metadata)
        self.metadata = {}

    def add_data(self, data, metadata=None):
        """Add data to the parameter set

        Parameters
        ----------
        data : numpy.ndarray
            one or more parameter sets. It must either be 1D or 2D, with the
            first dimension the number of parameter sets (K), and the second
            the number of elements (N): K x N
        metadata : object, optional
            the provided object will be stored in in the metadata dict and can
            be received with the ID that is returned. If multiple (K) datasets
            are added at ones, provide a list of objects with len K.

        Returns
        -------
        int, ID
            ID which can be used to access the parameter set

        Examples
        --------

        >>> # suppose that grid is a fully initialized grid oject with 100
            # elements
            parman = ParMan(grid)
            #
            one_data_set = np.ones(100)
            cid = parman.add_data(one_data_set)
            print(parman.parsets[cid])
            two_data_sets = np.ones((2, 100))
            cids = parman.add_data(two_data_sets)
            print(cids)
        [0, ]
        [1, 2]

        """
        subdata = np.atleast_2d(data)

        # we try to accommodate transposed input
        if subdata.shape[1] != self.grid.nr_of_elements:
            if subdata.shape[0] == self.grid.nr_of_elements:
                subdata = subdata.T
            else:
                raise Exception(
                    'Number of values does not match the number of '
                    'elements in the grid'
                )

        # now make sure that metadata can be zipped with the subdata
        K = subdata.shape[0]
        if metadata is not None:
            if K > 1:
                if(not isinstance(
                   metadata, (list, tuple)) or len(metadata) != K):
                    raise Exception('metadata does not fit the provided data')
            else:
                # K == 1
                metadata = [metadata, ]

        if metadata is None:
            metadata = [None for i in range(0, K)]

        return_ids = []
        for dataset, meta in zip(subdata, metadata):
            cid = self._get_next_index()
            self.parsets[cid] = dataset
            self.metadata[cid] = meta
            return_ids.append(cid)

        if len(return_ids) == 1:
            return return_ids[0]
        else:
            return return_ids

    def load_from_rho_file(self, filename):
        """Convenience function that loads two parameter sets from a rho.dat
        file, as used by CRMod for forward resistivity/phase models.

        Parameters
        ----------
        filename: string, file path
            filename of rho.dat file

        Returns
        -------
        cid_mag: int
            ID of magnitude parameter set
        cid_phase: int
            ID of phase parameter set

        """
        data = np.loadtxt(filename, skiprows=1)
        cid_mag = self.add_data(data[:, 0])
        cid_pha = self.add_data(data[:, 1])
        return cid_mag, cid_pha

    def load_inv_result(self, filename, columns=2, is_log10=False):
        """Load one parameter set from a rho*.mag or rho*.pha file produced by
        CRTomo.

        Parameters
        ----------
        filename : str, file path
            Filename to loaded data from
        columns : int or iterable of ints, optional
            column(s) to add to the manager. Defaults to 2 (third column).
        is_log10 : bool, optional
            If set to True, assume values to be in log10 and convert the
            imported values to linear before importing

        Returns
        -------
        pid : int or list of ints
            ID(s) of parameter set

        """
        assert os.path.isfile(filename)
        try:
            iterator = iter(columns)
        except TypeError:
            # not iterable
            iterator = [columns, ]

        data = np.loadtxt(filename, skiprows=1)
        pid_list = []
        for column in iterator:
            if is_log10:
                data[:, column] = 10 ** data[:, column]
            pid = self.add_data(data[:, column])
            pid_list.append(pid)

        if len(pid_list) == 1:
            return pid_list[0]
        else:
            return pid_list

    def load_model_from_file(self, filename, columns=0):
        """Load one parameter set from a file which contains one value per line

        No row is skipped.

        Parameters
        ----------
        filename : string, file path
            Filename to loaded data from
        columns : int or iterable of ints, optional
            column(s) to add to the manager. Defaults to 0 (first column)

        Returns
        -------
        pid : int or list of ints
            ID(s) of parameter set
        """
        try:
            iterator = iter(columns)
        except TypeError:
            # not iterable
            iterator = [columns, ]

        assert os.path.isfile(filename)
        data = np.loadtxt(filename).squeeze()
        # make sure data is NxC, where C is the number of datasets (1-?)
        if len(data.shape) == 1:
            data = data[:, np.newaxis]
        # check that all columns can be added
        assert data.shape[1] > np.max(columns)

        pid_list = []
        for column in iterator:
            pid = self.add_data(data[:, column])
            pid_list.append(pid)

        if len(pid_list) == 1:
            return pid_list[0]
        else:
            return pid_list

    def load_from_sens_file(self, filename):
        """Load real and imaginary parts from a sens.dat file generated by
        CRMod

        Parameters
        ----------
        filename: string
            filename of sensitivity file

        Returns
        -------
        nid_re: int
            ID of real part of sensitivities
        nid_im: int
            ID of imaginary part of sensitivities
        """
        sens_data = np.loadtxt(filename, skiprows=1)
        nid_re = self.add_data(sens_data[:, 2])
        nid_im = self.add_data(sens_data[:, 3])
        return nid_re, nid_im

    def save_to_rho_file(self, filename, cid_mag, cid_pha=None):
        """Save one or two parameter sets in the rho.dat forward model format

        Parameters
        ----------
        filename: string (file path)
            output filename
        cid_mag: int
            ID of magnitude parameter set
        cid_pha: int, optional
            ID of phase parameter set. If not set, will be set to zeros.

        """
        mag_data = self.parsets[cid_mag]
        if cid_pha is None:
            pha_data = np.zeros(mag_data.shape)
        else:
            pha_data = self.parsets[cid_pha]

        with open(filename, 'wb') as fid:
            fid.write(
                bytes(
                    '{0}\n'.format(self.grid.nr_of_elements),
                    'utf-8',
                )
            )
            np.savetxt(
                fid,
                np.vstack((
                    mag_data,
                    pha_data,
                )).T,
                fmt='%f %f'
            )

    def add_empty_dataset(self, value=1):
        """Create an empty data set. Empty means: all elements have the same
        value.

        Parameters
        ----------
        value: float, optional
            which value to assign all element parameters. Default is one.
        """
        subdata = np.ones(self.grid.nr_of_elements) * value
        pid = self.add_data(subdata)
        return pid

    def _clean_pid(self, pid):
        """if pid is a number, don't do anything. If pid is a list with one
        entry, strip the list and return the number. If pid contains more than
        one entries, do nothing.
        """
        if isinstance(pid, (list, tuple)):
            if len(pid) == 1:
                return pid[0]
            else:
                return pid
        return pid

    def modify_area(self, pid, xmin, xmax, zmin, zmax, value):
        """Modify the given dataset in the rectangular area given by the
        parameters and assign all parameters inside this area the given value.

        Partially contained elements are treated as INSIDE the area, i.e., they
        are assigned new values.

        Parameters
        ----------
        pid: int
            id of the parameter set to modify
        xmin: float
            smallest x value of the area to modify
        xmax: float
            largest x value of the area to modify
        zmin: float
            smallest z value of the area to modify
        zmin: float
            largest z value of the area to modify
        value: float
            this value is assigned to all parameters of the area

        Examples
        --------

        >>> import crtomo.tdManager as CRtdm
            tdman = CRtdm.tdMan(
                    elem_file='GRID/elem.dat',
                    elec_file='GRID/elec.dat',
            )
            pid = tdman.parman.add_empty_dataset(value=1)
            tdman.parman.modify_area(
                    pid,
                    xmin=0,
                    xmax=2,
                    zmin=-2,
                    zmin=-0.5,
                    value=2,
            )
            fig, ax = tdman.plot.plot_elements_to_ax(pid)
            fig.savefig('out.png')

        """
        area_polygon = shapgeo.Polygon(
            ((xmin, zmax), (xmax, zmax), (xmax, zmin), (xmin, zmin))
        )
        self.modify_polygon(pid, area_polygon, value)

    def modify_polygon(self, pid, polygon, value):
        """Modify parts of a parameter set by setting all parameters located
        in, or touching, the provided :class:`shapely.geometry.Polygon`
        instance.

        Parameters
        ----------
        pid: int
            id of parameter set to vary
        polygon: :class:`shapely.geometry.Polygon` instance
            polygon that determines the area to modify
        value: float
            value that is assigned to all elements in the polygon

        Examples
        --------
        >>> import shapely.geometry
            polygon = shapely.geometry.Polygon((
                (2, 0), (4, -1), (2, -1)
            ))
            tdman.parman.modify_polygon(pid, polygon, 3)

        """
        # create grid polygons
        grid_polygons = []
        for x, z in zip(self.grid.grid['x'], self.grid.grid['z']):
            coords = [(a, b) for a, b in zip(x, z)]
            grid_polygons.append(
                shapgeo.Polygon(coords)
            )

        # now determine elements in area
        elements_in_area = []
        for nr, element in enumerate(grid_polygons):
            if polygon.contains(element):
                elements_in_area.append(nr)
            elif polygon.equals(element):
                elements_in_area.append(nr)
            elif polygon.crosses(element):
                elements_in_area.append(nr)
                # only take crossed elements with at least A % overlap
                # int_area = polygon.intersect(element).area
                # print('overlap: ',
                #       int_area,
                #       element.area,
                #       element.area / int_area
                #       )

        # change the values
        pid_clean = self._clean_pid(pid)
        self.parsets[pid_clean][elements_in_area] = value

    def modify_pixels(self, pid, pixels, new_value):
        """Replace the value of a given pixel by a new one.

        Parameters
        ----------
        pid : int
            Parset id
        pixel : int|itearble
            Pixel index (zero-indexed)
        new_value : float
            New value that is assigned to the pixel
        """
        if not isinstance(pixels, Iterable):
            pixels = [pixels, ]
        for pixel in pixels:
            self.parsets[pid][pixel] = new_value

    def extract_points(self, pid, points):
        """Extract values at certain points in the grid from a given parameter
        set. Cells are selected by interpolating the centroids of the cells
        towards the line using a "nearest" scheme.

        Note that data is only returned for the points provided. If you want to
        extract multiple data points along a line, defined by start and end
        point, use the **extract_along_line** function.

        Parameters
        ----------
        pid : int
            The parameter id to extract values from
        points : Nx2 numpy.ndarray
            (x, y) pairs

        Returns
        -------
        values : numpy.ndarray (n x 1)
            data values for extracted data points
        """
        xy = self.grid.get_element_centroids()
        data = self.parsets[pid]

        iobj = spi.NearestNDInterpolator(xy, data)
        values = iobj(points)
        return values

    def extract_along_line(self, pid, xy0, xy1, N=10):
        """Extract parameter values along a given line.

        Parameters
        ----------
        pid : int
            The parameter id to extract values from
        xy0 : tuple
            A tupe with (x,y) start point coordinates
        xy1 : tuple
            A tupe with (x,y) end point coordinates
        N : integer, optional
            The number of values to extract along the line (including start and
            end point). Default: 10

        Returns
        -------
        values : numpy.ndarray (n x 3)
            data values for extracted data points. First column: x-coordinates,
            second column: z-coordinates, third column: extracted values
        """
        assert N >= 2
        xy0 = np.array(xy0).squeeze()
        xy1 = np.array(xy1).squeeze()
        assert xy0.size == 2
        assert xy1.size == 2

        # compute points
        points = [(x, y) for x, y in zip(
            np.linspace(xy0[0], xy1[0], N), np.linspace(xy0[1], xy1[1], N)
        )]
        result = self.extract_points(pid, points)

        results_xyv = np.hstack((
            points,
            result[:, np.newaxis]
        ))
        return results_xyv

    def extract_polygon_area(self, pid, polygon_points):
        """Extract all data points whose element centroid lies within the given
        polygon.

        Parameters
        ----------
        pid : int
            The parameter id to extract values from
        polygon_points : list of (x,y) floats
            list of points that form the polygon

        Returns
        -------
        elements_in_polygon_array : numpy.ndarray
            list with elements numbers (zero-indexed, starting from 0) which
            lie within the polygon
        values : list
            the data values corresponding to the elements in
            elements_in_polygon
        """
        polygon = shapgeo.Polygon(polygon_points)
        xy = self.grid.get_element_centroids()
        elements_in_polygon = []

        for nr, point in enumerate(xy):
            if shapgeo.Point(point).within(polygon):
                elements_in_polygon.append(nr)

        values = self.parsets[pid][elements_in_polygon]
        elements_in_polygon_array = np.array(elements_in_polygon)

        return elements_in_polygon_array, values

    def create_parset_with_gaussian_anomaly(
            self, center, width, max_value, background):
        """

        Parameters
        ----------
        center : [float, float]
            Center of the anomaly
        width : float| [float, float]
            The spatial width of the anomaly (x/z directions). Equivalent to
            the standard deviations of the underlying distribution.
        max_value : float
            The maximum value that the anomaly is normalized to
        background : float
            Background value

        Returns
        -------
        pid : int
            ID of newly created parameter set

        """
        pid = self.add_empty_dataset(background)
        self.add_gaussian_anomaly_to_parset(
            pid, center, np.sqrt(width), max_value
        )
        return pid

    def add_gaussian_anomaly_to_parset(
            self, pid, center, width, max_value):
        """

        Parameters
        ----------
        pid : int
            Id of parameter set
        center : [float, float]
            Center of the anomaly
        width : float| [float, float]
            The spatial width of the anomaly (x/z directions). Equivalent to
            the standard deviations of the underlying distribution.
        max_value : float
            The maximum value that the anomaly is normalized to

        """
        xy = self.grid.get_element_centroids()
        # generates a 2D Gaussian distribution with
        covariances = np.array(width) ** 2
        rv = multivariate_normal(
            center,
            covariances,
        )
        grid_values = rv.pdf(xy)

        if isinstance(width, (list, tuple)):
            assert len(width) == 2, "need either one number or two"
            std_x = width[0]
            std_z = width[1]
        else:
            std_x = width
            std_z = width
        # peak value of stretched normal distribution
        peak_x = 1 / (np.sqrt(2 * np.pi) * std_x)
        peak_z = 1 / (np.sqrt(2 * np.pi) * std_z)
        # resulting 2D peak
        peak_value = peak_x * peak_z
        grid_values /= peak_value
        grid_values *= max_value
        self.parsets[pid] = self.parsets[pid] + grid_values

    def add_gaussian_line(
                self, pid,
                p0,
                length,
                rotation,
                spacing,
                anomaly_peak,
                anomaly_std,
            ):
        xpos = np.arange(p0[0], length, step=spacing)
        zpos = np.ones_like(xpos) * p0[1]

        # rotate
        xr = xpos * np.cos(
            rotation * np.pi / 180) - zpos * np.sin(rotation * np.pi / 180)
        zr = zpos * np.cos(
            rotation * np.pi / 180) + xpos * np.sin(rotation * np.pi / 180)

        for x, z in zip(xr, zr):
            self.add_gaussian_anomaly_to_parset(
                pid,
                [x, z],
                width=anomaly_std,
                max_value=anomaly_peak,
            )

        return xr, zr

    def add_2d_cos_anomaly_line(
            self, pid, p0, anomaly_width, anomaly_height, peak_value,
            area='only_one_y',
            only_one_y_line=True, only_one_x_line=False, whole_mesh=False):
        """Add one line of cos(x)cos(y) anomalies to a given parameter set. The
        wavelength refers to half a period, with the maximum of the anomaly
        centered on p0=[x0, z0].

        Parameters
        ----------
        whole_mesh : bool, optional
            If True, then fill the whole mesh with a cos(x)cos(y) pattern.


        """
        coords = self.grid.get_element_centroids()
        norm = 2 * np.pi

        wavelength_x = anomaly_width * 2
        wavelength_z = anomaly_height * 2

        if area == 'all':
            indices = np.arange(0, coords.shape[0]).astype(int)
        elif area == 'only_one_y':
            boundary_y_min = p0[1] - (anomaly_height / 2)
            boundary_y_max = p0[1] + (anomaly_height / 2)
            # restrict to to within a rectangular area
            indices = np.where(
                (coords[:, 1] > boundary_y_min) &
                (coords[:, 1] < boundary_y_max)
            )
        elif area == 'only_one_x':
            boundary_x_min = p0[0] - (anomaly_width / 2)
            boundary_x_max = p0[0] + (anomaly_width / 2)
            # restrict to to within a rectangular area
            indices = np.where(
                (coords[:, 0] > boundary_x_min) &
                (coords[:, 0] < boundary_x_max)
            )

        anomaly = np.cos(
            (coords[indices, 0] - p0[0]) * norm / wavelength_x
        ) * np.cos(
            (coords[indices, 1] - p0[1]) * norm / wavelength_z
        ) * peak_value

        paradd = np.zeros(coords.shape[0])
        paradd[indices] += anomaly.squeeze()

        self.parsets[pid] += paradd

    def add_checkerboard_pattern(
            self, pid, p0, anomaly_width, anomaly_height, peak_value,
            ):
        """

        Note that if p0 is larger than a few anomaly sizes its possible that
        the checkerboard pattern will only be applied to parts of the mesh!

        """
        xlims, zlims = self.grid.get_minmax()

        x_positions = np.arange(
            p0[0],
            np.abs(xlims[1] - xlims[0]) + 4 * anomaly_width, anomaly_width * 2
        ) - (
            anomaly_width * 2
        ) * np.ceil(np.abs(xlims[0] - p0[0]) / (anomaly_width * 2))

        depth_sign = np.sign(zlims[0] - zlims[1])
        z_positions = np.arange(
            p0[1],
            (zlims[0] - zlims[1]) +
            depth_sign * 4 * anomaly_height, depth_sign * anomaly_height * 2
        )
        print(z_positions)
        offset = (anomaly_height * 2) * np.ceil(np.abs(zlims[0] - p0[1]) / (
            anomaly_height * 2))
        print(offset)

        coords = self.grid.get_element_centroids()
        paradd = np.zeros(coords.shape[0])

        for pos_x in x_positions:
            for pos_z in z_positions:
                indices = np.where(
                    (coords[:, 0] >= pos_x) &
                    (coords[:, 0] <= pos_x + anomaly_width) &
                    (coords[:, 1] <= pos_z) &
                    (coords[:, 1] >= (pos_z - anomaly_height))
                )
                paradd[indices] = peak_value

        self.parsets[pid] += paradd



#!/usr/bin/env python
# -*- coding: utf-8 -*-
'''
END DOCUMENTATION
'''
import numpy as np
import os
from optparse import OptionParser
import crtomo.plotManager as CRPlot
import crtomo.grid as CRGrid
import matplotlib.pyplot as plt
import matplotlib
import math
import reda.main.units as units
import crtomo.mpl as mpl_style


def handle_options():
    '''Handle options.
    '''
    parser = OptionParser()
    parser.set_defaults(cmaglin=False)
    parser.set_defaults(single=False)
    parser.set_defaults(alpha_cov=False)
    parser.add_option('-x',
                      '--xmin',
                      dest='xmin',
                      help='Minium X range',
                      type='float',
                      )
    parser.add_option('-X',
                      '--xmax',
                      dest='xmax',
                      help='Maximum X range',
                      type='float',
                      )
    parser.add_option('-z',
                      '--zmin',
                      dest='zmin',
                      help='Minium Z range',
                      type='float',
                      )
    parser.add_option('-Z',
                      '--zmax',
                      dest='zmax',
                      help='Maximum Z range',
                      type='float',
                      )
    parser.add_option('-c',
                      '--column',
                      dest='column',
                      help='column to plot of input file',
                      type='int',
                      default=2,
                      )
    parser.add_option('-u',
                      '--unit',
                      dest='xunit',
                      help='Unit of length scale, typically meters (m) ' +
                      'or centimeters (cm)',
                      metavar='UNIT',
                      type='str',
                      default='m',
                      )
    parser.add_option("--alpha_cov",
                      action="store_true",
                      dest="alpha_cov",
                      help="use coverage for transparency",
                      )
    parser.add_option('--cbtiks',
                      dest='cbtiks',
                      help="Number of CB tiks",
                      type=int,
                      metavar="INT",
                      default=3,
                      )
    parser.add_option("--cmaglin",
                      action="store_true",
                      dest="cmaglin",
                      help="linear colorbar for magnitude",
                      )
    parser.add_option('--mag_vmin',
                      dest='mag_vmin',
                      help='Minium of colorbar',
                      type='float',
                      )
    parser.add_option('--mag_vmax',
                      dest='mag_vmax',
                      help='Maximum of colorbar',
                      type='float',
                      )
    parser.add_option('--pha_vmin',
                      dest='pha_vmin',
                      help='Minium of colorbar',
                      type='float',
                      )
    parser.add_option('--pha_vmax',
                      dest='pha_vmax',
                      help='Maximum of colorbar',
                      type='float',
                      )
    parser.add_option('--real_vmin',
                      dest='real_vmin',
                      help='Minium of colorbar',
                      type='float',
                      )
    parser.add_option('--real_vmax',
                      dest='real_vmax',
                      help='Maximum of colorbar',
                      type='float',
                      )
    parser.add_option('--imag_vmin',
                      dest='imag_vmin',
                      help='Minium of colorbar',
                      type='float',
                      )
    parser.add_option('--imag_vmax',
                      dest='imag_vmax',
                      help='Maximum of colorbar',
                      type='float',
                      )

    (options, args) = parser.parse_args()
    return options


def check_minmax(plotman, cid, xmin, xmax, zmin, zmax, vmin, vmax):
    if xmin is None:
        xmin = plotman.grid.grid['x'].min()
    if xmax is None:
        xmax = plotman.grid.grid['x'].max()
    if zmin is None:
        zmin = plotman.grid.grid['z'].min()
    if zmax is None:
        zmax = plotman.grid.grid['z'].max()
    if isinstance(cid, int):
            subdata = plotman.parman.parsets[cid]
    else:
            subdata = cid
    if vmin is None:
        vmin = subdata.min()
    if vmax is None:
        vmax = subdata.max()

    return xmin, xmax, zmin, zmax, vmin, vmax


class overview_plot():

    def __init__(self, title, liste, alpha, unit, opt):
        self.title = title
        self.dirs = liste
        N = len(self.dirs)
        self.rows = math.ceil(N/4)
        self.columns = 4
        self.cbunit = units.get_label(unit)

        self.load_grid(alpha)
        self.create_figure(opt)
        self.cm(unit)

    def cm(self, unit):
        if unit == 'log_rho':
            self.cm = 'viridis'
        elif unit == 'phi':
            self.cm = 'plasma'
        elif unit == 'log_real':
            self.cm = 'viridis_r'
        elif unit == 'log_imag':
            self.cm = 'plasma_r'
        else:
            print('No colorbar defined')
            exit()

    def create_figure(self, opt):
        self.getfigsize(opt)
        self.fig, self.axs = plt.subplots(self.rows,
                                          ncols=4,
                                          figsize=(self.sizex,
                                                   self.sizez))
        plt.suptitle(self.title, fontsize=18)
        plt.subplots_adjust(wspace=4, hspace=4, top=5)

    def getfigsize(self, opt):
        '''calculate appropriate sizes for the subfigures
        '''
        if opt.xmin is None:
            opt.xmin = self.plotman.grid.grid['x'].min()
        if opt.xmax is None:
            opt.xmax = self.plotman.grid.grid['x'].max()
        if opt.zmin is None:
            opt.zmin = self.plotman.grid.grid['z'].min()
        if opt.zmax is None:
            opt.zmax = self.plotman.grid.grid['z'].max()
        if np.abs(opt.zmax - opt.zmin) < np.abs(opt.xmax - opt.xmin):
            self.sizex = 2 / 2.54
            self.sizez = self.sizex * (
                    np.abs(opt.zmax - opt.zmin) / np.abs(opt.xmax - opt.xmin))
        else:
            self.sizez = 2 / 2.54
            self.sizex = 0.5 * self.sizez * (
                    np.abs(opt.xmax - opt.xmin) / np.abs(opt.zmax - opt.zmin))
            print('schmal')
        # add 1 inch to accommodate colorbar
        self.sizex += 4 * .5
        self.sizex *= 4
        self.sizez *= self.rows
        self.sizez += 5

    def save(self):
        self.fig.tight_layout()
        self.fig.savefig('sd_' + self.title + '.png', dpi=300)

    def load_grid(self, alpha):
        '''Load grid and calculate alpha values from the coverage/2.5.
        '''
        grid = CRGrid.crt_grid(self.dirs[0] + '/grid/elem.dat',
                               self.dirs[0] + '/grid/elec.dat')
        self.plotman = CRPlot.plotManager(grid=grid)

        name = self.dirs[0] + '/inv/coverage.mag'
        content = np.genfromtxt(name, skip_header=1,
                                skip_footer=1, usecols=([2]))
        abscov = np.abs(content)
        if alpha:
            normcov = np.divide(abscov, 2.5)
            normcov[np.where(normcov > 1)] = 1
            mask = np.subtract(1, normcov)
            self.alpha = self.plotman.parman.add_data(mask)
        else:
            self.alpha = self.plotman.parman.add_data(np.ones(len(abscov)))


class subplot():

    def __init__(self, ov_plot, title, typ='cmplx'):
        self.title = title
        self.type = typ
        self.plotman = ov_plot.plotman

    def load_data(self, opt):
        os.chdir(self.title)
        # get iteration
        linestring = open('exe/inv.lastmod', 'r').readline().strip()
        linestring = linestring.replace('\n', '')
        linestring = linestring.replace('../', '')
        linestring = linestring.replace('mag', '')
        # open data file
        name = linestring + self.type

        if self.type == 'mag':
            try:
                self.data = np.loadtxt(name, skiprows=1,
                                       usecols=([opt.column]))
            except:
                raise ValueError('Given column to open does not exist.')
        if self.type == 'pha':
            try:
                self.data = np.loadtxt(name, skiprows=1, usecols=([2]))
            except:
                raise ValueError('No phase data to open.')
        os.chdir('..')

    def plot_data(self, ov_plot, opt, i, j):
        self.ax = ov_plot.axs[i, j]
        # add data to plotman
        if opt.cmaglin and self.type == 'mag':
            self.cid = self.plotman.parman.add_data(np.power(10, self.data))
            ov_plot.cbunit = 'rho'
        else:
            self.cid = self.plotman.parman.add_data(self.data)
        # handle options
        cblabel = units.get_label(ov_plot.cbunit)
        zlabel = 'z [' + opt.xunit + ']'
        xlabel = 'x [' + opt.xunit + ']'
        if self.type == 'mag':
            vmin = opt.mag_vmin
            vmax = opt.mag_vmax
        elif self.type == 'pha':
            vmin = opt.pha_vmin
            vmax = opt.pha_vmax
        elif self.type == 'imag':
            vmin = opt.imag_vmin
            vmax = opt.imag_vmax
        else:
            vmin = opt.real_vmin
            vmax = opt.real_vmax

        xmin, xmax, zmin, zmax, vmin, vmax = check_minmax(ov_plot.plotman,
                                                          self.cid,
                                                          opt.xmin, opt.xmax,
                                                          opt.zmin, opt.zmax,
                                                          vmin, vmax,
                                                          )
        # plot
        fig, ax, cnorm, cmap, cb = ov_plot.plotman.plot_elements_to_ax(
               cid=self.cid,
               cid_alpha=ov_plot.alpha,
               ax=self.ax,
               xmin=xmin,
               xmax=xmax,
               zmin=zmin,
               zmax=zmax,
               cblabel=cblabel,
               cbnrticks=opt.cbtiks,
               title=self.title[3:],
               zlabel=zlabel,
               xlabel=xlabel,
               plot_colorbar=True,
               cmap_name=ov_plot.cm,
               cbmin=vmin,
               cbmax=vmax,
               )


def calc_complex(mag, pha):
    ''' Calculate real and imaginary part of the complex conductivity from
    magnitude and phase in log10.
    '''
    complx = [10 ** m * math.e ** (1j * p / 1e3) for m, p in zip(mag, pha)]
    real = [math.log10((1 / c).real) for c in complx]
    imag = []
    for c in complx:
        if ((1 / c).imag) == 0:
            imag.append(math.nan)
        else:
            i = math.log10(abs((1 / c).imag))
            imag.append(i)
    return real, imag


def plot_cmplx(options, freq_dirs, ov_mag):
    # init overview plotsit='log_rho',
    ov_pha = overview_plot(title='Phase',
                           liste=freq_dirs,
                           alpha=options.alpha_cov,
                           unit='phi',
                           opt=options,
                           )
    ov_real = overview_plot(title='Real Part',
                            liste=freq_dirs,
                            alpha=options.alpha_cov,
                            unit='log_real',
                            opt=options,
                            )
    ov_imag = overview_plot(title='Imaginary Part',
                            liste=freq_dirs,
                            alpha=options.alpha_cov,
                            unit='log_imag',
                            opt=options,
                            )
    # create figure

    i = 0
    j = 0

    # plot each subplot
    for sub in np.arange(ov_mag.rows * ov_mag.columns):
        try:
            mag = subplot(ov_plot=ov_mag,
                          title=freq_dirs[sub],
                          typ='mag',
                          )
            pha = subplot(ov_plot=ov_pha,
                          title=freq_dirs[sub],
                          typ='pha',
                          )
            imag = subplot(ov_plot=ov_imag,
                           title=freq_dirs[sub],
                           typ='imag',
                           )
            real = subplot(ov_plot=ov_real,
                           title=freq_dirs[sub],
                           typ='real',
                           )

            mag.load_data(options)
            pha.load_data(options)
            real.data, imag.data = calc_complex(mag.data, pha.data)

            mag.plot_data(ov_mag, options, i//4, j)
            pha.plot_data(ov_pha, options, i//4, j)
            imag.plot_data(ov_imag, options, i//4, j)
            real.plot_data(ov_real, options, i//4, j)
        except:
            # no subplot needed
            ov_mag.axs[i//4, j].axis('off')
            ov_pha.axs[i//4, j].axis('off')
            ov_imag.axs[i//4, j].axis('off')
            ov_real.axs[i//4, j].axis('off')
        i = i + 1
        j = j + 1
        if j == 4:
            j = 0

    os.chdir('..')
    # save plots
    ov_mag.save()
    ov_pha.save()
    ov_real.save()
    ov_imag.save()


def plot_dc(options, freq_dirs, ov_mag):
    # init overview plots
    # create figure

    i = 0
    j = 0

    # plot each subplot
    for sub in np.arange(ov_mag.rows * ov_mag.columns):
        try:
            mag = subplot(ov_plot=ov_mag,
                          title=freq_dirs[sub],
                          typ='mag')

            mag.load_data(options)

            mag.plot_data(ov_mag, options, i//4, j)
        except:
            # no subplot needed
            ov_mag.axs[i//4, j].axis('off')
        i = i + 1
        j = j + 1
        if j == 4:
            j = 0

    os.chdir('..')
    # save plots
    ov_mag.save()


def main():
    # options
    options = handle_options()
    matplotlib.style.use('default')
    mpl_style.general_settings()

    # directories to plot
    os.chdir('invmod')
    freq_dirs = os.listdir('.')
    freq_dirs.sort()

    ov_mag = overview_plot(title='Magnitude',
                           liste=freq_dirs,
                           alpha=options.alpha_cov,
                           unit='log_rho',
                           opt=options,
                           )
    # check for phase
    if os.path.isfile(ov_mag.dirs[0] + '/inv/rho00.pha'):
        phase = True
    else:
        phase = False

    if phase:
        plot_cmplx(options, freq_dirs, ov_mag)
    else:
        plot_dc(options, freq_dirs, ov_mag)


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Plot a nice histogram of element angles to plot_element_angles.jpg. The
script expects an elem.dat file in the present working directory.

Examples
--------

    >>> ls -1
    elec.dat
    elem.dat
    >>> grid_plot_elem_angles
    >>> ls -1
    elec.dat
    elem.dat
    plot_element_angles.jpg


"""
import os

import crtomo.grid as CRGrid


def main():
    if not os.path.isfile('elem.dat'):
        raise Exception('elem.dat not found!')

    grid = CRGrid.crt_grid()
    grid.load_elem_file('elem.dat')
    fig, ax = grid.analyze_internal_angles(return_plot=True)
    fig.savefig('plot_element_angles.jpg', dpi=300)



#!/usr/bin/env python
# *-* coding: utf-8 *-*
"""Rotate a given grid clockwise

Note that, at the moment, the grid is always rotated around the origin (0,0).
"""
from optparse import OptionParser
import numpy as np
import crtomo.grid as CRGrid
import math


def handle_cmd_options():
    parser = OptionParser()
    parser.add_option('-e', "--elem", dest="elem_file", type="string",
                      help="elem.dat file (default: elem.dat)",
                      default="elem.dat")

    # parser.add_option("-x", "--center_x", dest="center_x", type="float",
    #                   help="Center around which to rotate (X-coordiante)",
    #                   default=0.0)
    # parser.add_option("-y", "--center_y", dest="center_y", type="float",
    #                   help="Center around which to rotate (Y-coordiante)",
    #                   default=0.0)

    parser.add_option("-a", "--angle", dest="angle_deg", type="float",
                      help="Rotation angle (in degrees, default: 90)",
                      default=90.0)
    parser.add_option("-o", "--output", dest="output",
                      help="Output file (default: elem_rot.dat)",
                      metavar="FILE", default="elem_rot.dat")

    (options, args) = parser.parse_args()
    return options


def rotmat(alpha):
    """Rotate around z-axis
    """
    R = np.array(((np.cos(alpha), -np.sin(alpha)),
                 (np.sin(alpha), np.cos(alpha))))

    return R


def rotate_nodes(xy, cx, cy, angle_rad):
    rot_xy = []
    R = rotmat(angle_rad)

    for vector in xy:
        rot_xy.append(R.dot(vector))
    rot_xy_array = np.array(rot_xy)
    return rot_xy_array


def main():
    options = handle_cmd_options()
    # put in dummy center coordinates
    options.center_x = 0.0
    options.center_y = 0.0

    grid = CRGrid.crt_grid()
    grid.load_elem_file(options.elem_file)
    rotated_nodes = rotate_nodes(
        grid.nodes['raw'][:, 1:3],
        options.center_x,
        options.center_y,
        math.radians(options.angle_deg)
    )
    grid.nodes['raw'][:, 1:3] = rotated_nodes
    grid.save_elem_file(options.output)


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
"""Thils file set ups matplotlib plot functions for the whole package.

Import all necessary Matplotlib modules and set default options
To use this module, import * from it:

Examples
--------

    >>> from crtomo.mpl_setup import *

"""
import sys
already_loaded = 'matplotlib' in sys.modules

# just make sure we can access matplotlib as mpl
import matplotlib as mpl

if(not already_loaded):
    mpl.use('Agg')

# general settings
mpl.rcParams["lines.linewidth"] = 2.0
mpl.rcParams["lines.markeredgewidth"] = 3.0
mpl.rcParams["lines.markersize"] = 3.0
mpl.rcParams["font.size"] = 8
# mpl.rcParams['font.sans-serif'] = 'Droid Sans'

mpl.rcParams['font.family'] = 'Open Sans'
# mpl.rcParams['font.weight'] = 400
mpl.rcParams['mathtext.default'] = 'regular'

# mpl.rcParams['font.family'] = 'Droid Sans'

mpl.rcParams['text.usetex'] = False

mpl.rc(
    'text.latex',
    preamble=''.join((
        # r'\usepackage{droidsans} ',
        # r'\usepackage[T1]{fontenc} ',
        r'\usepackage{sfmath} \renewcommand{\rmfamily}{\sffamily}',
        r'\renewcommand\familydefault{\sfdefault} ',
        # r'\usepackage{mathastext} '
    ))
)


mpl.use('Agg')

import matplotlib.pyplot as plt
plt

import mpl_toolkits.axes_grid1 as axes_grid1
axes_grid1


def mpl_get_cb_bound_next_to_plot(ax):
    """ Return the coordinates for a colorbar axes next to the provided axes
    object. Take into account the changes of the axes due to aspect ratio
    settings.

    Parts of this code are taken from the transforms.py file from matplotlib

    Important: Use only AFTER fig.subplots_adjust(...)

    Examples
    --------

    >>> cb_pos = mpl_get_cb_bound_next_to_plot(fig.axes[15])
        ax1 = fig.add_axes(cb_pos, frame_on=True)
        cmap = mpl.cm.jet_r
        norm = mpl.colors.Normalize(vmin=float(23), vmax=float(33))
        # cmap = pm.cmap
        # norm = pm.norm
        cb1 = mpl.colorbar.ColorbarBase(
            ax1,
            cmap=cmap,
            norm=norm,
            orientation='vertical'
        )
        cb1.locator = mpl.ticker.FixedLocator([23, 28, 33])
        cb1.update_ticks()
        cb1.ax.artists.remove(cb1.outline)    # remove framei
    """
    position = ax.get_position()

    figW, figH = ax.get_figure().get_size_inches()
    fig_aspect = figH / figW
    box_aspect = ax.get_data_ratio()
    pb = position.frozen()
    pb1 = pb.shrunk_to_aspect(box_aspect, pb, fig_aspect).bounds

    ax_size = ax.get_position().bounds

    xdiff = (ax_size[2] - pb1[2]) / 2
    ydiff = (ax_size[3] - pb1[3]) / 2

    # the colorbar is set to 0.01 width
    sizes = [ax_size[0] + xdiff + ax_size[2] + 0.01,
             ax_size[1] + ydiff,
             0.01,
             pb1[3]]

    return sizes


def mpl_get_cb_bound_below_plot(ax):
    """ Return the coordinates for a colorbar axes below the provided axes
    object. Take into account the changes of the axes due to aspect ratio
    settings.

    Important: Use only AFTER fig.subplots_adjust(...)

    """
    position = ax.get_position()

    figW, figH = ax.get_figure().get_size_inches()
    fig_aspect = figH / figW
    box_aspect = ax.get_data_ratio()
    pb = position.frozen()
    pb1 = pb.shrunk_to_aspect(box_aspect, pb, fig_aspect).bounds

    ax_size = ax.get_position().bounds

    # xdiff = (ax_size[2] - pb1[2]) / 2
    # ydiff = (ax_size[3] - pb1[3]) / 2

    # the colorbar is set to 0.01 width
    sizes = [ax_size[0], ax_size[1], pb1[2], 0.03]

    return sizes
